import universe

__author__="peterman"
__date__ ="$May 16, 2010 1:31:45 PM$"

class Player(universe.GameObject):

    def __init__(self, player_id):
        self.set_id(player_id)

    def alive(self):
        return self.get_emperor_name()

    def set_moo2data(self, moo2data):
        self.__moo2data = moo2data

    def get_moo2data(self):
        return self.__moo2data

    def set_emperor_name(self, emperor_name):
        self.__emperor_name = emperor_name

    def get_emperor_name(self):
        return self.__emperor_name

    def set_race_name(self, race_name):
        self.__race_name = race_name

    def get_race_name(self):
        return self.__race_name

    def set_picture(self, picture):
        self.__picture = picture

    def get_picture(self):
        return self.__picture

    def set_color(self, color):
        self.__color = color

    def get_color(self):
        return self.__color

    def set_personality(self, personality):
        self.__personality = personality

    def get_personality(self):
        return self.__personality

    def set_objective(self, objective):
        self.__objective = objective

    def get_objective(self):
        return self.__objective

    def set_tax_rate(self, tax_rate):
        self.__tax_rate = tax_rate

    def get_tax_rate(self):
        return self.__tax_rate

    def set_bc(self, bc):
        self.__bc = bc

    def get_bc(self):
        return self.__bc

    def raise_bc(self):
        self.set_bc(self.get_bc() + self.get_bc_income())

    def set_total_frighters(self, total_frighters):
        self.__total_frighters = total_frighters

    def get_total_frighters(self):
        return self.__total_frighters

    def set_used_frighters(self, used_frighters):
        self.__used_frighters = used_frighters

    def get_used_frighters(self):
        return self.__used_frighters

    def set_command_points(self, command_points):
        self.__command_points = command_points

    def get_command_points(self):
        return self.__command_points

    def set_industry(self, industry):
        self.__industry = industry

    def get_industry(self):
        return self.__industry

    def set_research(self, research):
        self.__research = research

    def get_research(self):
        return self.__research

    def add_research(self, research):
        self.__research += research

    def set_food(self, food):
        self.__food = food

    def add_food(self, food):
        self.__food += food

    def get_food(self):
        return self.__food

    def set_bc_income(self, bc_income):
        self.__bc_income = bc_income

    def get_bc_income(self):
        return self.__bc_income

    def set_research_progress(self, research_progress):
        self.__research_progress = research_progress

    def get_research_progress(self):
        return self.__research_progress

    def raise_research(self):
        self.__research_progress += self.get_research()

    def reseatch_completed(self):
        return self.get_research_progress() >= self.get_research_costs()

    def set_research_area(self, research_area):
        self.__research_area = research_area

    def get_research_area(self):
        return self.__research_area

    def set_research_item(self, research_item):
        self.__research_item = research_item

    def get_research_item(self):
        return self.__research_item

    def set_research_costs(self, research_costs):
        self.__research_costs = research_costs

    def get_research_costs(self):
        return self.__research_costs

    def set_research_turns_left(self, research_turns_left):
        if (research_turns_left) < 0:
            self.__research_turns_left = 0
        else:
            self.__research_turns_left = research_turns_left

    def get_research_turns_left(self):
        return self.__research_turns_left

    def set_racepicks(self, racepicks):
        self.__racepicks = racepicks

    def get_racepicks(self):
        return self.__racepicks

    def set_racepick_item(self, item, value):
        self.__racepicks[item] = value

    def get_racepick_item(self, item):
        return self.__racepicks[item]

    def set_known_techs(self, known_techs):
        self.__known_techs = known_techs

    def get_known_techs(self):
        return self.__known_techs

    def add_known_technology(self, tech_id):
        self.__known_techs.append(tech_id)

    def knows_technology(self, tech_id):
        return tech_id in self.__known_techs

    def set_prototypes(self, prototypes):
        self.__prototypes = prototypes

    def get_prototypes(self):
        return self.__prototypes

    def add_prototype(self, prototype):
        self.__prototypes.append(prototype)

    def set_tributes(self, tributes):
        self.__tributes = tributes

    def get_tributes(self):
        return self.__tributes

    def add_tribute(self, tribute):
        self.__tributes.append(tribute)

    def update_research_areas(self, research_areas):
        self.__research_areas = research_areas

    def list_research_areas(self):
        return self.__research_areas

    def print_debug(self):
        print
        print("=== player_id = %i ===" % self.get_id())
        print("     race     = %s" % self.get_race_name())
        print("     emperor  = %s" % self.get_emperor_name())
        print("     color    = %i" % self.get_color())
        print
        print("     food     = %i" % self.get_food())
        print
        print("     research = %i" % self.get_research())
        print("     research_area = %i" % self.get_research_area())
        print("     research_item = %i" % self.get_research_item())
        print("     research_costs = %i" % self.get_research_costs())
        print("     research_progress = %i" % self.get_research_progress())
        print("     research_turns_left = %i" % self.get_research_turns_left())

    def print_research_debug(self):
        print("research_item = %i, research_area = %i, research_progress = %i, " % (self.get_research_item(), self.get_research_area(), self.get_research_progress()))



import pygame
import os
import hashlib

DEBUG	= 0

#COLORKEY = (253, 254, 255)
COLORKEY = (0xdf, 0x32, 0xef)



def read_byte(data, offset):
    """returns 1 byte on given offset"""
    return ord(data[offset])
# /read_byte

def read_signed_byte(data, offset):
    """returns 1-byte signed integer on given offset"""
    v = ord(data[offset])
    if v > 0x7f:
        v -= 0x100
    return v
# /read_signed_byte

def read_char(buffer, offset):
    return ord(buffer[offset])
# /read_char

def read_short_int(data, offset):
    """returns 2-byte unsigned integer on given offset"""
    return ord(data[offset]) + (ord(data[offset+1]) << 8)
# /read_short_int

def read_signed_short_int(data, offset):
    """returns 2-byte unsigned integer on given offset"""
    si = ord(data[offset]) + (ord(data[offset+1]) << 8)
    if si > 0x7fff:
	return si - 0x10000
    else:
	return si
# /read_short_int

def read_long_int(data, offset):
        """returns 4-byte unsigned integer on given offset"""
        return ord(data[offset]) + (ord(data[offset+1]) << 8) + (ord(data[offset+2]) << 16) + (ord(data[offset+3]) << 24)
# /read_long_int

def read_string(data, offset, length):
    """returns string on given offset"""
    return data[offset:offset + length].rstrip(chr(0) + chr(1) + chr(2) + chr(3)).split(chr(0))[0]
# /read_string


def debug_output(s):
    print(s)

#def moo2_draw_image_frame(surface, picture, frame, globalPalette, x=0, y=0):
def moo2_draw_image_frame(surface, picture, frame, localPalette, x=0, y=0, DEBUG = None):
#    localPalette = []
#    debug_output("	Drawing frame #%i (frame offset is 0x%x)" % (frame, picture['offsets'][frame]))
#    print "* globalPalette: " + str(globalPalette)
#    print "globalPalette ID: " + str(id(globalPalette))
#    localPalette = copy.deepcopy(globalPalette)
#    localPalette = globalPalette[:]
#    localPalette = globalPalette
#    print "localPalette ID: " + str(id(localPalette))
#    print
#    print "* localPalette: " + str(localPalette)
#    print "tested palette ... before applycation of local palette: " + str(localPalette)
#    print "tested palette ... before applycation of local palette: " + str(localPalette)
#    return
#    print picture['palette'][1]

# dump set the black color as transparent, let's see if it will break something :-O

    px = pygame.PixelArray(surface)

    pos = 12 + (len(picture['offsets']) << 2)	# offset * 4

    if picture['f_junction']:
        debug_output("		TODO: junction")

    if picture['f_functional_color']:
        debug_output("		TODO: functional color")

    if picture['f_fill_background']:
	# FIXME: COLORKEY is hardcoded, should be determined somehow?
        surface.fill(COLORKEY)
        surface.set_colorkey(COLORKEY)
	
    if picture['f_no_compression']:
        debug_output("		TODO: no compression")

    if picture['f_internal_palete']:
#        debug_output("		Image has local palette")
        colorShift = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
        pos += 2
        localColors = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
        pos += 2
#        debug_output("		Local palette shift: " + str(colorShift))
#        debug_output("		Local palette colors: " + str(localColors))
        for i in range(localColors):
            a = ord(picture['data'][pos])
            r = ord(picture['data'][pos+1]) * 4
            g = ord(picture['data'][pos+2]) * 4
            b = ord(picture['data'][pos+3]) * 4
#            debug_output("* localPalette COLOR BEFORE update: " + str(localPalette[colorShift + i]))
#            debug_output("* globalPalette COLOR BEFORE update: " + str(globalPalette[colorShift + i]))
            localPalette[colorShift + i]['alpha'] = a
            localPalette[colorShift + i]['rgb'] = (r << 16) + (g << 8) + b
#            debug_output("* localPalette [%i] COLOR AFTER update: %s" % (i, str(localPalette[colorShift + i])))
#            debug_output("* globalPalette [%i] COLOR AFTER: %s" % (i, str(globalPalette[colorShift + i])))
            pos += 4
#        debug_output()
#        debug_output("* localPalette UPDATED: " + str(localPalette))
#        debug_output("* globalPalette: " + str(globalPalette))
    X = x
    pos = picture['offsets'][frame]
    if (ord(picture['data'][pos]) == 1) and (ord(picture['data'][pos+1]) == 0):
#		print "heading 0x01 0x00 looks ok"
        pos += 2
        startY = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
#		print "startY: " + str(startY)
        Y = y + startY
        pos += 2
        while pos < picture['offsets'][frame+1]:
            pixelCount = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
#	    print "pixelCount: %i" % pixelCount
#	    if pixelCount == 1:
#		print "color at [0,0] %i" % px[0][0]
            pos += 2
#			print "pixelCount: " + str(pixelCount)
            if pixelCount > 0:
                xIndent = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
                X += xIndent
                pos += 2
#				print "xIndent: " + str(xIndent)
                for i in range(pixelCount):
                    pixel = ord(picture['data'][pos])
#		    if localPalette[pixel]['alpha']:
#                	px[X][Y] = localPalette[pixel]['rgb']
#                	pixel = ord(picture['data'][pos])
#			print "		got alpha: %i" % localPalette[pixel]['alpha']
#		    else:
            	    px[X][Y] = localPalette[pixel]['rgb']
#                	pixel = ord(picture['data'][pos])
            	    pos += 1
# put-pixel onto surface
#                    px[X][Y] = localPalette[pixel]['rgb']
                    X += 1
                if (pixelCount % 2) == 1:
                    pos += 1
#				print
            else:
                yIndent = ord(picture['data'][pos]) + (ord(picture['data'][pos+1]) << 8)
                pos += 2
                if yIndent != 1000:
                    Y += yIndent
                    X = x
                else:
#					pygame.display.flip()
#					pos = 999999999
#                    print
                    return

# end func moo2_draw_image_frame

def load_surface(picture, picture_frame, local_palette, colorkey = None):
#    debug_output("@ load_surface")
#    debug_output("	lbx_source_filename: %s" % lbx_source_filename)
#    debug_output("	lbx_file_index: %i" % lbx_file_index)
#    debug_output("	picture_frame: %i" % picture_frame)
#    debug_output("	colorkey: %s" % colorkey)
#    debug_output("")
    """
    """

    surface = pygame.Surface((picture['width'], picture['height']))
#    if lbx_source_filename == "APP_PICS.LBX":
#	moo2_draw_image_frame(surface, picture, picture_frame, local_palette, 0, 0, True)
#	moo2_draw_image_frame(surface, picture, picture_frame, local_palette)
#    else:
#	moo2_draw_image_frame(surface, picture, picture_frame, local_palette)
    moo2_draw_image_frame(surface, picture, picture_frame, local_palette)
    if colorkey is not None:
#	print("	setting colorkey!")
	surface.set_colorkey(colorkey)
    return surface
# end func load_surface


class Archive():

#    __filename = None
#    __filesize = 0
#    __version = None

#    __info = None

#    __files_info = None

    def __init__(self, filename, md5sum = ""):
#        print("lbx::Archive::__init__() ... MD5 = %s ... filename = %s" % (md5sum, filename))
        self.__filename = filename
        self.__filesize = os.path.getsize(filename)
	self.__file = None
        self.open()
        self.__md5_expected = md5sum
        self.__count_md5_hexdigest()
        self.__load_info()

    def get_filename(self):
        return self.__filename

    def open(self):
        if self.__file is None:
            self.__file = open(self.__filename, 'rb')

    def close(self):
        if self.__file:
            self.__file.close()
            self.__file = None

    def __count_md5_hexdigest(self):
        checksum = hashlib.md5()
        for one_line in self.__file.readlines():
            checksum.update(one_line)
        self.__md5_real = str(checksum.hexdigest())

    def md5_hexdigest(self):
        return self.__md5_real

    def md5_hexdigest_expected(self):
        return self.__md5_expected

    def check_md5(self):
        return self.__md5_expected == self.__md5_real

    def read(self, n):
        return self.__file.read(n)

    def seek(self, offset):
        return self.__file.seek(offset)

    def __read_int_2b_le(self):
        """reads 2-byte little-endian integer"""
	n = self.read(2)
	return ord(n[0]) + (ord(n[1]) << 8)

    def __read_int_4b_le(self):
        """reads 4-byte little-endian integer"""
        n = self.read(4)
        return ord(n[0]) + (ord(n[1]) << 8) + (ord(n[2]) << 16) + (ord(n[3]) << 24)

    def check_lbx_signature(self, signature):
        """returns True if the given string is a correct LBX signature"""
	return signature == "\xAD\xFE\x00\x00"

    def __load_info(self):
        """loads archived files descriptors"""
        self.seek(0)
	n = self.__read_int_2b_le()
	if self.check_lbx_signature(self.read(4)):
#                print "good signature"
                self.__version = self.read(2)

            	self.__info = []
		off2 = self.__read_int_4b_le()
		for i in range(n):
			off1 = off2
			if i == (n):
				off2 = filesize
			else:
				off2 = self.__read_int_4b_le()
			size = off2 - off1
			self.__info.append({'size': size, 'offset': off1})
                return True
	else:
#                print "bad signature"
		return False

    def get_info(self):
        """returns archived files descriptors"""
        return self.__info

    def has_file(self, i):
#        print "LbxArchive::has_file ... i = " + str(i)
        return (i >= 0) and (i <= len(self.__info))

    def get_file_info(self, i):
        if self.has_file(i):
            return self.__info[i]
        else:
            return None

    def read_file(self, i):
        f_info = self.get_file_info(i)
        if not f_info:
            return None
        else:
            self.seek(f_info['offset'])
            return self.read(f_info['size'])

    def read_picture(self, i):
        data = self.read_file(i)
        if not data:
            return None
        else:
            flags = ord(data[10]) + (ord(data[11]) << 8)
            picture = {
                'width':                ord(data[0]) + (ord(data[1]) << 8),
                'height':               ord(data[2]) + (ord(data[3]) << 8),
                'frames':               ord(data[6]) + (ord(data[7]) << 8),
                'frame_delay':          ord(data[8]) + (ord(data[9]) << 8),
                'flags':                flags,
                'flags_hex':            hex(flags),
                'f_junction':           flags & 8192,
                'f_internal_palete':    flags & 4096,
                'f_functional_color':   flags & 2048,
                'f_fill_background':    flags & 1024,
                'f_no_compression':     flags & 256,
                'offsets':              [],
                'data':                 data
            }

            for i in range(picture['frames'] + 1):
                    ii = 12 + (i * 4)
                    picture['offsets'].append(ord(data[ii]) + (ord(data[ii+1]) << 8) + (ord(data[ii+2]) << 16) + (ord(data[ii+3]) << 24))

            return picture

    def load_surface(self, picture, frame, local_palette, color_key):
	return load_surface(picture, frame, local_palette, color_key)

    def get_surface(self, picture_id, picture_frame, local_palette, color_key = None):
        return self.load_surface(self.read_picture(picture_id), picture_frame, local_palette, color_key)

    def read_palette(self, i):
        data = self.read_file(i)
        if not data:
            return None
        else:
            palette = []
            for i in range(256):
                a = ord(data[i*4])
                r = ord(data[(i*4)+1]) * 4
                g = ord(data[(i*4)+2]) * 4
                b = ord(data[(i*4)+3]) * 4
                palette.append({
                    'alpha': a,
                    'rgb': (r << 16) + (g << 8) + b
                })
            return palette

# end class Archive



B_ATMOSPHERE_RENEWER		=  5
B_CORE_WASTE_DUMP		= 13
B_BIOSPHERES			= 15
#B_HOLO_SIMULATOR		= 20
#B_MARINE_BARRACKS		= 22
B_GRAVITY_GENERATOR		= 25
B_STOCK_EXCHANGE		= 29
B_RECYCLOTRON			= 33
B_ROBOTIC_FACTORY		= 34
B_SPACEPORT			= 39


"""
B_NO_BUILDING			=  0
B_ALIEN_CONTROL_CENTER		=  1
B_ARMOR_BARRACKS		=  2
B_ARTEMIS_SYSTEM_NET		=  3
B_ASTRO_UNIVERSITY		=  4
B_AUTOLAB			=  6
B_AUTOMATED_FACTORY		=  7
B_BATTLESTATION			=  8
B_CAPITOL			=  9
B_CLONING_CENTER		= 10
B_COLONY_BASE			= 11
B_DEEP_CORE_MINE		= 12
B_DIMENSIONAL_PORTAL		= 14
B_FOOD_REPLICATORS		= 16
B_GAIA_TRANSFORMATION		= 17
B_CURRENCY_EXCHANGE		= 18
B_GALACTIC_CYBERNET		= 19
B_HYDROPONIC_FARM		= 21
B_BARRIER_SHIELD		= 23
B_FLUX_SHIELD			= 24
B_MISSILE_BASE			= 26
B_GROUND_BATTERIES		= 27
B_RADIATION_SHIELD		= 28
B_SUPERCOMPUTER			= 30
B_PLEASURE_DOME			= 31
B_POLLUTION_PROCESSOR		= 32
B_RESEARCH_LAB			= 35
B_ROBO_MINER_PLANT		= 36
B_SOIL_ENRICHMENT		= 37
B_SPACE_ACADEMY			= 38
B_STAR_BASE			= 40
B_STAR_FORTRESS			= 41
B_STELLAR_CONVERTER		= 42
B_SUBTERRANEAN_FARMS		= 43
B_TERRAFORMING			= 44
B_WARP_INTERDICTOR		= 45
B_WEATHER_CONTROLLER		= 46
B_FIGHTER_GARRISON		= 47
B_ARTIFICIAL_PLANET		= 48
"""

BUILDINGS = {

      0: {
        'name':     		"",
        'tech':     		0,
      },

      1: {
        'name':     		"Alien Control Center",
        'tech':     		5,
      },

      2: {
        'name':     		"Armor Barracks",
        'tech':     		14,
      },

      3: {
        'name':     		"Artemis System Net",
        'tech':     		15,
      },

      4: {
        'name':     		"Astro University",
        'tech':     		18,
	'farmer_bonus':		+1,
	'worker_bonus':		+1,
	'scientist_bonus':	+1,
      },

      5: {
        'name':     		"Atmosphere Renewer",
        'tech':     		19,
      },

      6: {
        'name':     		"Autolab",
        'tech':     		21,
	'research':		+30,
      },

      7: {
        'name':     		"Automated Factory",
        'tech':     		22,
	'worker_bonus':		+1,
	'industry':		+5,
      },

      8: {
        'name':     		"Battlestation",
        'tech':     		27,
        'replaces':             (-1, 40),
      },

      9: {
        'name':     		"Capitol",
        'tech':     		32,
      },

     10: {
        'name':     		"Cloning Center",
        'tech':     		39,
      },

     11: {
        'name':     		"Colony Base",
        'tech':     		40,
      },

     12: {
        'name':     		"Deep Core Mine",
        'tech':     		49,
	'worker_bonus':		+3,
	'industry':		+15,
      },

     13: {
        'name':     		"Core Waste Dump",
        'tech':     		50,
        'replaces':             (5, 32)
      },

     14: {
        'name':     		"Dimensional Portal",
        'tech':     		52,
      },

     15: {
        'name':     		"Biospheres",
        'tech':     		61,
      },

     16: {
        'name':     		"Food Replicators",
        'tech':     		68,
      },

     17: {
        'name':                 "Gaia Transformation",
        'tech':                 74,
      },

     18: {
        'name':                 "Currency Exchange",
        'tech':                 75,
      },

     19: {
        'name':                 "Galactic Cybernet",
        'tech':                 76,
        'research':             +15,
      },

     20: {
        'name':                 "Holo Simulator",
        'tech':                 86,
        'morale_bonus':         20, # percent
      },

     21: {
        'name':     		"Hydroponic Farm",
        'tech':     		87,
	'food_production':	+2,
      },

     22: {
        'name':                             "Marine Barracks",
        'tech':                             103,
        'government_morale_bonus':          [20, 20, 20, 20, 0, 0, 0], # percent
      },

     23: {
        'name':                 "Barrier Shield",
        'tech':                 129,
        'replaces':             (-1, 28, 24),
      },

     24: {
        'name':                 "Flux Shield",
        'tech':                 130,
        'replaces':             (-1, 28)
      },

     25: {
        'name':                 "Gravity Generator",
        'tech':                 131,
      },

     26: {
        'name':                 "Missile Base",
        'tech':                 132,
      },

     27: {
        'name':                 "Ground Batteries",
        'tech':                 133,
      },

     28: {
        'name':                 "Radiation Shield",
        'tech':                 134,
      },

     29: {
        'name':                 "Stock Exchange",
        'tech':                 135,
      },

     30: {
        'name':     		"Supercomputer",
        'tech':     		136,
	'scientist_bonus':	+2,
	'research':		+10,
      },

     31: {
        'name':                 "Pleasure Dome",
        'tech':                 141,
        'morale_bonus':         30, # percent
      },

     32: {
        'name':                 "Pollution Processor",
        'tech':                 142,
      },

     33: {
        'name':                 "Recyclotron",
        'tech':                 152,
      },

     34: {
        'name':                 "Robotic Factory",
        'tech':                 154,
        'industry_by_minerals': [5, 8, 10, 15, 20],
      },

     35: {
        'name':     		"Research Lab",
        'tech':     		155,
	'scientist_bonus':	+1,
	'research':		+5,
      },

     36: {
        'name':     		"Robo Miner Plant",
        'tech':     		156,
	'worker_bonus':		+2,
	'industry':		+10,
      },

     37: {
        'name':     		"Soil Enrichment",
        'tech':     		162,
      },

     38: {
        'name':     		"Space Academy",
        'tech':     		163,
      },

     39: {
        'name':     		"Spaceport",
        'tech':     		164,
      },

     40: {
        'name':     		"Star Base",
        'tech':     		168,
      },

     41: {
        'name':     		"Star Fortress",
        'tech':     		169,
        'replaces':             (40, 8),
      },

     42: {
        'name':     		"Stellar Converter",
        'tech':     		174,
      },

     43: {
        'name':     		"Subterranean Farms",
        'tech':     		178,
        'food_production':      +4,
      },

     44: {
        'name':     		"Terraforming",
        'tech':     		183,
      },

     45: {
        'name':     		"Warp Interdictor",
        'tech':     		197,
      },

     46: {
        'name':     		"Weather Controller",
        'tech':     		198,
	'farmer_bonus':		+2,
      },

     47: {
        'name':     		"Fighter Garrison",
        'tech':     		67,
      },

     48: {
        'name':     		"Artifical Planet",
        'tech':     		16,
      },

    90: {
        'name':     		"? BUILDING 90 ?",
        'tech':     		0,
      },

    96: {
        'name':     		"? BUILDING 96 ?",
        'tech':     		0,
      },

    105: {
        'name':     		"? BUILDING 105 ?",
        'tech':     		0,
      },

    106: {
        'name':     		"? BUILDING 106 ?",
        'tech':     		0,
      },

    107: {
        'name':     		"? BUILDING 107 ?",
        'tech':     		0,
      },

    108: {
        'name':     		"? BUILDING 108 ?",
        'tech':     		0,
      },

    112: {
        'name':     		"? BUILDING 112 ?",
        'tech':     		0,
      },

    116: {
        'name':     		"? BUILDING 116 ?",
        'tech':     		0,
      },

    117: {
        'name':     		"? BUILDING 117 ?",
        'tech':     		0,
      },

    123: {
        'name':     		"? BUILDING 123 ?",
        'tech':     		0,
      },

    124: {
        'name':     		"? BUILDING 124 ?",
        'tech':     		0,
      },

    128: {
        'name':     		"? BUILDING 128 ?",
        'tech':     		0,
      },

    141: {
        'name':     		"Transport",
        'tech':     		189,
      },

    142: {
        'name':     		"Outpost Ship",
        'tech':     		109,
      },

    143: {
        'name':     		"Colony Ship",
        'tech':     		41,
      },

    153: {
        'name':     		"? BUILDING 153 ?",
        'tech':     		0,
      },

    214: {
        'name':     		"Freighter Fleet",
        'tech':     		69,
      },

    241: {
        'name':     		"? BUILDING 241 ?",
        'tech':     		0,
      },

    246: {
        'name':     		"Spy",
        'tech':     		166,
      },

    249: {
        'name':     		"Repeat",
        'tech':     		0,
        'extra':    		'repeat'
      },

    253: {
        'name':     		"Housing",
        'tech':     		0,
      },

    254: {
        'name':     		"Trade Goods",
        'tech':     		0,
      },
}



import socket
import threading
import time
import hashlib
import pickle
import game_socket

class Server():

    def __init__(self, host, port, game):
	self.set_name("unnamed")
	self.set_host(host)
	self.set_port(port)
	self.set_socket_buffer_size(4096)
	self.__game = game
	self.__players_status = {}
	self.__threads = {}
	self.__threads_next_turn = {}
	
	for player_id in range(self.max_players()):
	    self.__players_status[player_id] = 0
	    
    def set_name(self, name):
	self.__name = name
	
    def get_name(self):
	return self.__name
	    
    def spawn_server_socket(self):
        new_server_socket = game_socket.GameSocket(socket.socket(socket.AF_INET, socket.SOCK_STREAM))
        new_server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        new_server_socket.settimeout(1)
        new_server_socket.bind((self.__host, self.__port))
        new_server_socket.listen(2)
	return new_server_socket

    def log_info(self, message):
	print("# INFO: %s" % message)

    def log_debug(self, message):
	print("> DEBUG: %s" % message)

    def log_error(self, message):
	print("! ERROR: %s" % message)
	    
    def __set_status(self, status):
	if status not in ["init", "turn", "recount", "shutdown"]:
	    self.log_error("Server::set_status() ... bad status: %s" % status)
	else:
	    self.__status = status
#	self.log_info("Server::__set_status() ... status set to \"%s\"" % status)
    
    def __get_status(self):
	return self.__status
	    
    def set_turn(self):
        self.__set_status("turn")

    def set_recount(self):
        self.__set_status("recount")

    def set_shutdown(self):
        self.__set_status("shutdown")

    def check_turn(self):
        return self.__get_status() == "turn"

    def check_recount(self):
        return self.__get_status() == "recount"

    def check_shutdown(self):
        return self.__get_status() == "shutdown"

    def set_host(self, host):
        self.__host = host

    def get_host(self):
        return self.__host

    def set_port(self, port):
        self.__port = port

    def get_port(self):
        return self.__port

    def max_players(self):
	return 8;

    def set_socket_buffer_size(self, size):
	self.__socket_buffer_size = size

    def get_socket_buffer_size(self):
	return self.__socket_buffer_size

    def get_data_for_player(self, player_id):
        return self.__game.get_data_for_player(player_id)

    def send_data_to_game_client(self, client_socket, thread_id, data):
        s = pickle.dumps(data)
#	self.log_info("Server::send_data_to_game_client() ... %i bytes for thread %s" % (len(s), thread_id))
	client_socket.send(data)

    def set_next_turn(self, thread_id, player_id):
	self.__threads_next_turn[thread_id] = player_id

    def clean_next_turn(self, thread_id):
	self.set_next_turn(thread_id, -1)

    def wait_for_next_turn(self):
	pass
	
    def chech_next_turn(self):
	# chech_next_turn_multiplayer seems to work fine
	# but for now every game is supposed to be a single player
	return self.chech_next_turn_singleplayer()

    def chech_next_turn_singleplayer(self):
	# in single player human is always player_id 0
	for thread_id, player_id in self.__threads_next_turn.items():
	    if player_id  == 0:
		return True
	return False

    def chech_next_turn_multiplayer(self):
	pl_st = {}
	# all the clients must confirm next turn
	for thread_id, player_id in self.__threads_next_turn.items():
	    if player_id < 0:
		return False
	    else:
		pl_st[player_id] = True
	self.log_info("Server::chech_next_turn() ... pl_st = %s" % str(pl_st))
	# all players must confirm next turn
	return len(pl_st) == self.__game.count_players()

    def run_client_handler(self, client_socket, thread_id):
	"""
    
	"""
        self.log_info("thread_id: %s\n    STARTED\n" % (thread_id))
	PLAYER_ID = -1
        socket_down = {}
	while not self.check_shutdown():
#    	    data = client_socket.recv(self.get_socket_buffer_size())
    	    data = client_socket.recv()
    	    if not data:

                # an ugly detection if the remote connection disappeared
                # explain: there is a 1s timeout, so counter should not exceed even a low number within 1 second...
                ts = int(time.time())
                if socket_down.has_key(ts):
                    socket_down[ts] += 1
                else:
                    socket_down = {ts: 1}

                if socket_down[ts] > 10:
                    self.log_info("!!! Server::run_client_handler() ... socket_down[%i] = %i" % (ts, socket_down[ts]))
                    self.log_info("!!! remote socket is down, closing local socket in thread %s" % thread_id)
                    break
                    
    	    else:
        	self.log_info("data received from client # %s, PLAYER_ID = %i" % (thread_id, PLAYER_ID))

#        	if data[:8] == "shutdown":
#		    # unsecure, just to allow clean shutdown via telnet for development
#            	    self.log_info("received shutdown!")
#		    self.set_shutdown()
#		else:

#		    data = pickle.loads(data)
                ACTION = data['action']
                PARAMS = data['params']
		    
                self.log_info("thread_id: %s\n    PLAYER_ID: %i\n    ACTION: %s\n    PARAMS: %s\n" % (thread_id, PLAYER_ID, ACTION, str(PARAMS)))

                if ACTION == "LOGIN":
                    # SECURE ME!!!
                    PLAYER_ID = PARAMS['player_id']
                    self.log_info("thread \"%s\" ... PLAYER_ID set to %i" % (thread_id, PLAYER_ID ))

                elif ACTION == "LOGOUT":
                    break

                elif ACTION == "PING":
                    self.send_data_to_game_client(client_socket,  thread_id, "PONG")

                elif ACTION == "GET_NAME":
                    self.send_data_to_game_client(client_socket,  thread_id, self.get_name())

                elif ACTION == "GET_SERVER_STATUS":
                    self.send_data_to_game_client(client_socket, thread_id, self.__get_status())

                elif ACTION == "NEXT_TURN":
                    self.log_info("received NEXT_TURN from thread %s ... player_id = %i" % (thread_id, PLAYER_ID))
                    self.set_next_turn(thread_id, PLAYER_ID)
                    self.send_data_to_game_client(client_socket,  thread_id, "NEXT_TURN_ACK")

                elif PLAYER_ID < 0:
                    self.log_error("anonymous player sending actions!!!")

                elif ACTION == "FETCH_GAME_DATA":
                    self.send_data_to_game_client(client_socket, thread_id, self.get_data_for_player(PLAYER_ID))

                elif ACTION == "SET_RESEARCH":
                    research_item = PARAMS['tech_id']
                    if not self.__game.update_research(PLAYER_ID, research_item):
                        self.log_error("Game::set_research() failed ... PLAYER_ID = %i, research_item = %i" % (PLAYER_ID, research_item))
                else:
                    self.log_error("unknow action received from client: '%s'" % data)

        self.log_info("thread_id: %s\n    PLAYER_ID: %i\n    CLOSING SOCKET\n" % (thread_id, PLAYER_ID))
	client_socket.close()
	self.clean_next_turn(thread_id)

    def compose_thread_id(self, client_host, client_port):
	now = str(time.time())
	s = "%s:%i@%s" % (client_host, client_port, now)
	return "%s:%s:%i" % (hashlib.md5(s).hexdigest(), client_host, client_port)

    def spawn_thread(self, client_socket, client_host, client_port):
	thread_id = self.compose_thread_id(client_host, client_port)

        th = threading.Thread(target = self.run_client_handler, args=(client_socket, thread_id))
#	thread_ident = str(th.get_ident())
    	self.log_info("Server::spawn_thread() ... new thread spawned: %s" % thread_id)
	self.__threads_next_turn[thread_id] = -1
	self.__threads[thread_id] = th
        self.__threads[thread_id].setDaemon(True)
        self.__threads[thread_id].start()
	return thread_id

    def clean_thread(self, thread_id):
	self.log_info("* Server::clean_thread() ... cleaning thread # %s" % thread_id)
	self.__threads.pop(thread_id)
	self.__threads_next_turn.pop(thread_id)

    def show_next_turns(self):
	for thread_id, player_id in self.__threads_next_turn.items():
	    print("? next_turn: %s ... %i" % (thread_id, player_id))

    def debug_threads(self):
	if self.__threads:
	    print("# Threads:")
	    for thread_id in self.__threads:
		is_alive = self.__threads[thread_id].is_alive()
		self.log_debug("#    id = %s, is_alive = %i" % (thread_id, is_alive))

    def run(self):

        print("")
        print("PORT = %s" % self.__port)
        print("SOCKET_READ_BUFFER_SIZE = " + str(self.get_socket_buffer_size()))
        print "Awaiting connections..."
        print("")

	server_socket = self.spawn_server_socket()

	self.set_turn()

        while not self.check_shutdown():
            try:
                client_socket, (client_host, client_port) = server_socket.accept()
		thread_id = self.spawn_thread(client_socket, client_host, client_port)

            except socket.timeout:
                pass

#	    self.debug_threads()
#	    self.show_next_turns()

	    if self.__threads:
		clean_threads = []
		for thread_id in self.__threads:
		    is_alive = self.__threads[thread_id].is_alive()
		    if not is_alive:
			self.log_info("Server::run() ... thread %s is not alive, wil be cleaned" % thread_id)
			clean_threads.append(thread_id)

		# clean old threads
		for thread_id in clean_threads:
		    self.clean_thread(thread_id)

		if self.chech_next_turn():
		    self.log_info("got True from Server::chech_next_turn() >>> DOING NEXT TURN!")
		    self.__game.next_turn()
		    # clean next turn flags for all threads
		    for thread_id in self.__threads:
			self.clean_next_turn(thread_id)
#		else:
#		    self.log_info("# next turn not ready...")

        self.log_info("Closing serverSocket")
        server_socket.close()




__author__="peterman"
__date__ ="$Jan 7, 2009 8:44:49 PM$"

import pygame

import get_input
#import debug

#from rules import RULES

SHOW_TYPE = None

##
#       DRAW
##
def draw(GAME):
    global SHOW_TYPE

    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']
    FONTS	= GAME['FONTS']

    DATA        = GAME['DATA']
    STARS       = DATA['stars']

    DISPLAY.blit(IMAGES['LEADERS_SCREEN'], (0, 0))

    if SHOW_TYPE == 1:
	DISPLAY.blit(IMAGES['OFFICER.LBX']['buttons']['colony_leaders'], (7, 10))
    else:
	DISPLAY.blit(IMAGES['OFFICER.LBX']['buttons']['ship_officers'], (160, 10))


#    debug.debug_heroes(GAME)

#    HEROES = GAME['DATA']['heroes']
#    HEROES = DATA[["my_colony_leaders", "officers"][SHOW_TYPE]]
    HEROES = DATA[["colony_leaders", "officers"][SHOW_TYPE]]
    
    i = -1
    for hero_id in HEROES.keys():
        hero = HEROES[hero_id]
#        print(hero)
#	if (hero['player'] == 0) and (hero['type'] == SHOW_TYPE):
        i += 1
#	    print(hero['name'])
        DISPLAY.blit(IMAGES['OFFICER.LBX']['leaders'][hero['picture']], (13, 38 + (109 * i)))

        DISPLAY.blit(FONTS['font_12_bold'].render(hero['name'], 1, (0x78, 0x9c, 0xc0)), (125, 36 + (109 * i)))

        if hero['location'] == 0xffff:
            location = "Officer Pool"
        else:
            if SHOW_TYPE == 1:
                location = STARS[hero['location']].get_name()
            elif SHOW_TYPE == 0:
                location = "ship..."

        text_width, text_height = FONTS['font_11'].size(location)
        DISPLAY.blit(FONTS['font_11'].render(location, 1, (0x78, 0x9c, 0xc0)), (49 - (text_width / 2), 130 + (109 * i)))

        skills = []

        if hero['type'] == 0:		# ship leader skills
            if hero['special_skills'] & 4:
                skills.append(['fighter_pilot', "Fighter Pilot"])
            if hero['special_skills'] & 8:
                skills.append(['fighter_pilot', "Fighter Pilot*"])
            if hero['special_skills'] & 16:
                skills.append(['galactic_role', "Galactic Role"])
            if hero['special_skills'] & 32:
                skills.append(['galactic_role', "Galactic Role*"])
            if hero['special_skills'] & 64:
                skills.append(['helmsman', "Helmsman"])
            if hero['special_skills'] & 128:
                skills.append(['helmsman', "Helmsman*"])
            if hero['special_skills'] & 256:
                skills.append(['navigator', "Navigator"])
            if hero['special_skills'] & 512:
                skills.append(['navigator', "Navigator*"])
            if hero['special_skills'] & 1024:
                skills.append(['ordnance', "Ordnance"])
            if hero['special_skills'] & 2048:
                skills.append(['ordnance', "Ordnance*"])
            if hero['special_skills'] & 16384:
                skills.append(['weaponry', "Weaponry"])
            if hero['special_skills'] & 32768:
                skills.append(['weaponry', "Weaponry*"])

        elif hero['type'] == 1:		# colony leader skills
            if hero['special_skills'] & 16:
                skills.append(['financial_leader', "Financial Leader"])
            if hero['special_skills'] & 32:
                skills.append(['financial_leader', "Financial Leader*"])
            if hero['special_skills'] & 64:
                skills.append(['instructor', "Instructor"])
            if hero['special_skills'] & 128:
                skills.append(['instructor', "Instructor*"])
            if hero['special_skills'] & 256:
                skills.append(['labor_leader', "Labor Leader"])
            if hero['special_skills'] & 512:
                skills.append(['labor_leader', "Labor Leader*"])
            if hero['special_skills'] & 1024:
                skills.append(['medicine', "Medicine"])
            if hero['special_skills'] & 2048:
                skills.append(['medicine', "Medicine*"])
            if hero['special_skills'] & 4096:
                skills.append(['science_leader', "Science Leader"])
            if hero['special_skills'] & 8196:
                skills.append(['science_leader', "Science Leader*"])
            if hero['special_skills'] & 16384:
                skills.append(['spiritual_leader', "Spiritual Leader"])
            if hero['special_skills'] & 32768:
                skills.append(['spiritual_leader', "Spiritual Leader*"])
            if hero['special_skills'] & 65536:
                skills.append(['tactics', "Tactics"])
            if hero['special_skills'] & 131072:
                skills.append(['tactics', "Tactics*"])
	    
        if hero['common_skills'] & 1:	# commond skills
            skills.append(['assassin', "Assassin"])
        if hero['common_skills'] & 2:
            skills.append(['assassin', "Assassin*"])
        if hero['common_skills'] & 4:
            skills.append(['commando', "Commando"])
        if hero['common_skills'] & 8:
            skills.append(['commando', "Commando*"])
        if hero['common_skills'] & 16:
            skills.append(['diplomat', "Diplomat"])
        if hero['common_skills'] & 32:
            skills.append(['diplomat', "Diplomat*"])
        if hero['common_skills'] & 64:
            skills.append(['famous', "Famous"])
        if hero['common_skills'] & 128:
            skills.append(['famous', "Famous*"])
        if hero['common_skills'] & 256:
            skills.append(['megawealth', "Megawealth"])
        if hero['common_skills'] & 512:
            skills.append(['megawealth', "Megawealth*"])
        if hero['common_skills'] & 1024:
            skills.append(['operations', "Operations"])
        if hero['common_skills'] & 2048:
            skills.append(['operations', "Operations*"])
        if hero['common_skills'] & 4096:
            skills.append(['researcher', "Researcher"])
        if hero['common_skills'] & 8192:
            skills.append(['researcher', "Researcher*"])
        if hero['common_skills'] & 16384:
            skills.append(['spy_master', "Spy Master"])
        if hero['common_skills'] & 32768:
            skills.append(['spy_master', "Spy Master*"])
        if hero['common_skills'] & 65536:
            skills.append(['telepath', "Telepath"])
        if hero['common_skills'] & 131072:
            skills.append(['telepath', "Telepath*"])
        if hero['common_skills'] & 262144:
            skills.append(['trader', "Trader"])
        if hero['common_skills'] & 524288:
            skills.append(['trader', "Trader*"])

#	    skills = []
#	    skills.append(['91', "91"])
#	    skills.append(['92', "92"])
#	    skills.append(['108', "108"])

        skill_row = 0
        for skill in skills:
            y = 50 + (109 * i) + (17 * skill_row)
            DISPLAY.blit(IMAGES['OFFICER.LBX']['skill_icons'][skill[0]], (94, y))
            DISPLAY.blit(FONTS['font_13_bold'].render(skill[1], 1, (0x78, 0x9c, 0xc0)), (114, y))
            skill_row += 1
		
	    

    pygame.display.flip()
# end func draw

##
#       RUN
##
def run(GAME):
    global SHOW_TYPE

    draw(GAME)

    triggers = [
        {'action': "ESCAPE",        	'rect': pygame.Rect((544, 445), (70, 18))},
        {'action': "hire",         	'rect': pygame.Rect((319, 445), (70, 18))},
        {'action': "showColonyLeaders", 'rect': pygame.Rect((15, 14), (135, 15))},
        {'action': "showShipOfficers",	'rect': pygame.Rect((160, 14), (135, 15))}

    ]

    while True:
        event = get_input.get_input(triggers)
        action = event['action']

        if (action == "ESCAPE"):
            return

        if (action == "showColonyLeaders"):
	    if SHOW_TYPE == 0:
        	SHOW_TYPE = 1
		draw(GAME)

        if (action == "showShipOfficers"):
	    if SHOW_TYPE == 1:
        	SHOW_TYPE = 0
		draw(GAME)

        else:
            print "UNKNONW ACTION: " + action
    

if SHOW_TYPE is None:
    SHOW_TYPE = 1



import pygame

from lbx import load_surface
from game import *

import get_input

import colony_screen
import colony_build_screen

VIEW_SIZE  = 10

LIST_START = 0
LIST_SIZE  = 0

##
#       DRAW
##
def draw(GAME):
    global SCREEN
    global VIEW_SIZE
    global LIST_START
    global LIST_SIZE

    GUI		= GAME['GUI']
    DISPLAY	= GAME['DISPLAY']
    IMAGES	= GAME['IMAGES']
    PALETTES	= GAME['PALETTES']
    FONTS	= GAME['FONTS']

#    STARS	= GAME['DATA']['stars']
#    PLANETS	= GAME['DATA']['planets']
    COLONIES	= GAME['DATA']['colonies']
    
    PLAYERS	= GAME['DATA']['players']
    ME          = GAME['DATA']['me']

    GUI.draw_screen('COLSUM', DISPLAY)

    my_colonies = []

    for colony_id in COLONIES:
        col = COLONIES[colony_id]
        if (col.get_owner() == ME.get_id()) and (not col.is_outpost()):
#        if (COLONIES[i].owner != 0xff) and (COLONIES[i].is_outpost == 0):
#	    my_colonies.append(COLONIES[i])
	    my_colonies.append("%s:%i" % (COLONIES[colony_id].get_name(), colony_id))

    print(my_colonies)

    my_colonies.sort()
    for i in range(len(my_colonies)):
	colony_id = int(my_colonies[i].split(":")[1])
	my_colonies[i] = COLONIES[colony_id]

    LIST_SIZE = len(my_colonies)

    buttons = []

    for i in range(LIST_START, min(LIST_SIZE, LIST_START + VIEW_SIZE)):
	colony = my_colonies[i]
	colony_id	= colony.get_id()
	planet_id	= colony.get_planet_id()

	if planet_id == 0xffff:
	    print colony
	    continue

        y = 38 + (31 * (i - LIST_START))
	
	buttons.append({'action': "colony", 'colony_id': colony_id, 'rect': pygame.Rect((12, y), (84, 29))})
	buttons.append({'action': "colony_build", 'colony_id': colony_id, 'rect': pygame.Rect((513, y), (82, 29))})

	DISPLAY.blit(FONTS['font_10'].render(colony.get_name(), 1, (0x80, 0xA0, 0xBC)), (12, y + 6))

	for t in (0x02, 0x82, 0x03):
	    if t == 0x02:
		x = 101
        	icon = 1
	    elif t == 0x82:
		x = 236
            	icon = 3
	    elif t == 0x03:
		x = 378
            	icon = 5

	    c = len(colony.colonists[t])
	    if c < 5:
        	xx = 28
    	    else:
        	xx = 114 / c

    	    for ii in range(c):
		colonist = colony.colonists[t][ii]
		race = colonist['race']
		picture = PLAYERS[race].get_picture()
		DISPLAY.blit(IMAGES['RACEICON.LBX'][picture][icon], (x + (xx * ii), y))

    pygame.display.flip()
    return buttons
# /draw

##
#       RUN
##
def run(GAME):
    global VIEW_SIZE
    global LIST_START
    global LIST_SIZE
    

    triggers = [
        {'action': "ESCAPE",        'rect': pygame.Rect((544, 445), (70, 18))},
        {'action': "SCROLL_UP",      'rect': pygame.Rect((620, 16), (8, 18))},
        {'action': "SCROLL_DOWN",    'rect': pygame.Rect((620, 318), (8, 18))}
    ]

    triggers2 = draw(GAME)

    while True:
        event = get_input.get_input(triggers + triggers2)
        action = event['action']
	
        if (action == "ESCAPE"):
            return

	elif action == "SCROLL_UP":
	    if LIST_START > 0:
		LIST_START -= 1
		triggers2 = draw(GAME)

	elif action == "SCROLL_DOWN":
	    if LIST_START < (LIST_SIZE - VIEW_SIZE):
		LIST_START += 1
		triggers2 = draw(GAME)

	elif (action == "colony"):
    	    colony_screen.run(GAME, event['colony_id'])
	    triggers2 = draw(GAME)

        elif (action == "colony_build"):
    	    colony_build_screen.run(GAME, event['colony_id'])
	    triggers2 = draw(GAME)

        else:
            print "UNKNONW ACTION: " + action



GOVERNMENTS = {
    0:	{
	'name':		"Feudal",
	'morale':       0,
    },
    1:	{
	'name':		"Feudal 2",
	'morale':       0,
    },
    2:	{
	'name':		"Dictatorship",
	'morale':       -20,                    # -20%
    },
    3:	{
	'name':		"Imperium",
	'morale':       0,
    },
    4:	{
	'name':		"Democracy",
	'morale':       0,
    },
    5:	{
	'name':		"Democracy 2",
	'morale':       0,
    },
    6:	{
	'name':		"Unification",
	'morale':       0,
    },
    7:	{
	'name':		"Galactic Unification",
	'morale':       0,
    },
}




#import formulas

#from game import *
#import techtable

def bin(i):
    if i > 1:
	return bin(i >> 1) + str(i & 1)
    return str(i & 1)
# end func bin

def debug_stars(GAME):
    STARS	= GAME['DATA']['stars']
    for star in STARS:
	print "Star % 3i ... % 15s ... size = %i ... coordinates: [%i, %i]" % (star['id'], star['name'], star['size'], star['x'], star['y'])
#	print "		size: %i" % star['size']
#	print
    print
# end func debug_stars

def debug_colonies_population(GAME):
    DATA	= GAME['DATA']
    STARS	= DATA['stars']
    PLANETS	= DATA['planets']
    COLONIES	= DATA['colonies']
    PLAYERS	= DATA['players']
    
    print "planets: %i" % len(PLANETS)
    for i in range(len(COLONIES)):
	colony		= COLONIES[i]
	planet_id	= colony.planet_id
	if planet_id < len(PLANETS):
#	    print "planet_id: %i" % planet_id
	    planet	= PLANETS[planet_id]
	    if colony.is_outpost():
		print "Outpost %i ... %s %i" % (i, STARS[planet['parent_star']]['name'], PLANETS[planet_id]['num'])
	    else:
		max_populations = colony.get_max_populations(PLAYERS)
		max_population = max(max_populations)
		research = colony.research
		print "Colony %i ... %s %i" % (i, STARS[planet['parent_star']]['name'], PLANETS[planet_id]['num'])
		print "			population: %i / %i (loaded max population: %i)" % (colony['population'], max_population, colony['max_population'])
		print "			research: %i (loaded is: %i)" % (research, colony['total_research'])
		for k in research_data.keys():
		    print "				%s: %s" % (k, research_data[k])
#    		print "colony #%.3i : % 15s % 10i / % 2i" % (i, name, colony['population'], max_population)
#		print "	planet_id: %i" % colony['planet_id']
#		print "		terraformations: %i" % planet['terraformations']
#		farming		= formulas.get_colony_farming(planet, colony, PLAYERS)
#		print "	max_population: %i (counted = %i)" % (colony['max_population'], max(max_populations))
#		print "	population: %i" % colony['population']
#		print "	owner: %s" % PLAYERS[colony['owner']]['race']
#		print "	total_food: %i (counted = %i)" % (colony['total_food'], farming)
#		print "	total_production: %i (counted = %i, poluttion = %i) ... RESULT = %i" % (colony['total_production'], production, pollution, (production - pollution))
		print
# end func debug_colonies_population


def debug_players(GAME):

    DATA = GAME['DATA']
#    DISPLAY = GAME['DISPLAY']
#    IMAGES = GAME['IMAGES']
#    FONTS = GAME['FONTS']

#    tchst = ["non-reserchable", "unknown", "???", "KNOWN"]
    technames = techtable.get_technames()

    player = DATA['players'][0]

    print "%s emperor of %s" % (player['emperor'], player['race'])
    print "	objective: %i" % player['objective']
    print "	color: %i" % player['color']
    print "     known technologies:"
    for i in range(203):
#        techname = "?"
        status = player['technologies'][i]
	if status == 3:
    	    print "         #%.3i %s" % (i, technames[i])
	elif player['research_item'] and (i == player['research_item']-1):
    	    print "         #%.3i %s ... RESEARCHING" % (i, technames[i])
    print

#end func debug_clonies_popilation

def debug_heroes(GAME):
    HEROES = GAME['DATA']['heroes']
    for i in range(len(HEROES)):
        hero = HEROES[i]
#	if hero['player'] < 0xff:
	if hero['player'] == 0:
    	    print "hero %i" % i
	    for k in hero.keys():
		print "	%s: %s" % (k, hero[k])
	    print
# end func debug_heroes

def debug_ships(GAME):
    SHIPS = GAME['DATA']


    return
    DICTIONARY	= GAME['DICTIONARY']
    PLAYERS	= GAME['DATA']['players']
#    STARS	= GAME['DATA']['stars']
    SHIPS	= GAME['DATA']['ships']
    i = 0
    statuses	= ["orbiting", "traveling", "launching", "status3", "status4", "status5", "building", "status7"]
    for ship in SHIPS:
	if ship['status'] != 5:			# 5 = destroyed?
	    design = ship['design']
	    print "Ship % 3i ... % 15s ... coordinates: [%i, %i]" % (i, design['name'], ship['x'], ship['y'])
#		print "	design:"
	    print "		        name: %s" % design['name']
	    print "		        size: %i ... %s" % (design['size'], DICTIONARY['SHIP_SIZES'][design['size']])
#	    print "		        type: %i ... %s" % (design['type'], DICTIONARY['SHIP_TYPES'][design['type']])
#	    print "		      shield: %i ... %s" % (design['shield'], DICTIONARY['SHIELDS'][design['shield']])
#	    print "		       drive: %i ... %s" % (design['drive'], DICTIONARY['DRIVES'][design['drive']])
#	    print "		       speed: %i" % design['speed']
#	    print "		    computer: %i ... %s" % (design['computer'], DICTIONARY['COMPUTERS'][design['computer']])
#	    print "		       armor: %i ... %s" % (design['armor'], DICTIONARY['ARMORS'][design['armor']])
#	    print "		    specials: ..."
#	    print "		     weapons: ..."
#	    print "		     picture: %i" % design['picture']
#	    print "		     builder: %i" % design['builder']
#	    print "		combat_speed: %i" % design['combat_speed']
#	    print "		  build_date: %s" % stardate(design['build_date'])
#	    print "	data:"
	    print "		       owner: %i ... %s" % (ship['owner'], PLAYERS[ship['owner']]['race'])
	    if ship['owner'] < 8:
		print "		player color: %i" % PLAYERS[ship['owner']]['color']
	    print "		      status: %i ... %s" % (ship['status'], statuses[ship['status']])
#	    print "		 coordinates: %i, %i" % (ship['x'], ship['y'])
	    print "		    location: %i" % ship['location']
	    print "		  location_x: %i" % ship['location_x']
#	    print "		    location: %i ... %s" % (ship['location'], STARS[ship['location']]['name'])
#	    print "		  turns_left: %i" % ship['turns_left']
	    if ship['status'] == 2:
		location = ship['location']
		location_x = ship['location_x']
		result = (location_x << 8) + location
		print "			location: %i ... %s" % (location, bin(location))
		print "		      location_x: %i ... %s" % (location_x, bin(location_x))
		print "			  result: %i ... %s" % (result, bin(result))
	    print
	    i += 1
# end func debug_ships



import copy
import lbx

class Gui():

    def __init__(self, pg, default_font):
	self.__pygame = pg
	self.__pygame.init()
	self.__pygame.display.set_mode((640, 480), 0, 24)
	self.__default_font = default_font
	self.__images = {}
    # /__init__

    def get_fonts(self):
        return {
	    'font_10':		self.__pygame.font.SysFont(self.__default_font, 10),
	    'font_10_bold':	self.__pygame.font.SysFont(self.__default_font, 10, True),
	    'font_11':		self.__pygame.font.SysFont(self.__default_font, 11),
	    'font_11_bold':	self.__pygame.font.SysFont(self.__default_font, 11, True),
	    'font_12':		self.__pygame.font.SysFont(self.__default_font, 12),
	    'font_12_bold':	self.__pygame.font.SysFont(self.__default_font, 12, True),
	    'font_13':		self.__pygame.font.SysFont(self.__default_font, 13),
	    'font_13_bold':	self.__pygame.font.SysFont(self.__default_font, 13, True),
	    'font_14':		self.__pygame.font.SysFont(self.__default_font, 14),
	    'font_14_bold':	self.__pygame.font.SysFont(self.__default_font, 14, True),
	    'font_15':		self.__pygame.font.SysFont(self.__default_font, 15),
	    'font_15_bold':	self.__pygame.font.SysFont(self.__default_font, 15, True),
	    'font_16':		self.__pygame.font.SysFont(self.__default_font, 16),
	    'font_16_bold':	self.__pygame.font.SysFont(self.__default_font, 16, True),
	    'font_17':		self.__pygame.font.SysFont(self.__default_font, 17),
	    'font_17_bold':	self.__pygame.font.SysFont(self.__default_font, 17, True),
	    'font_18':		self.__pygame.font.SysFont(self.__default_font, 18),
	    'font_18_bold':	self.__pygame.font.SysFont(self.__default_font, 18, True),
	    'FONT3':		self.__pygame.font.SysFont(self.__default_font, 18)
	}
    # /get_fonts

    def load_lbx_archives(self, moo2_dir = "../moo2"):
	self.__lbx = {
            'APP_PICS.LBX':     lbx.Archive("%s/APP_PICS.LBX" % moo2_dir, "7b04085428c25c8d1ef26d18c45e10ff"),
            'BUFFER0.LBX':      lbx.Archive("%s/BUFFER0.LBX" % moo2_dir, "72061ddf5816d13287a720c0606ea845"),
            'COLBLDG.LBX':      lbx.Archive("%s/COLBLDG.LBX" % moo2_dir, "d4c75440f233eb3170f1d01719f4b915"),
            'COLONY2.LBX':      lbx.Archive("%s/COLONY2.LBX" % moo2_dir, "cc054705454ceae81c37ffd42a312412"),
            'COLPUPS.LBX':      lbx.Archive("%s/COLPUPS.LBX" % moo2_dir, "2ecad5faf06c64afb8124b162d21b253"),
            'COLSUM.LBX':       lbx.Archive("%s/COLSUM.LBX" % moo2_dir, "cb9be56155b2304abb92613ca5549980"),
            'COLSYSDI.LBX':     lbx.Archive("%s/COLSYSDI.LBX" % moo2_dir, "d2ad38fc93b6876246868b880acd7ec9"),
            'FONTS.LBX':        lbx.Archive("%s/FONTS.LBX" % moo2_dir, "51002fdf406bac511f649fff9c1f531c"),
            'GAME.LBX':         lbx.Archive("%s/GAME.LBX" % moo2_dir, "6ac9233d3c0221c2a01c24e7e995de63"),
            'IFONTS.LBX':       lbx.Archive("%s/IFONTS.LBX" % moo2_dir, "deb8a07d2446b725810904d1d73def87"),
            'INFO.LBX':         lbx.Archive("%s/INFO.LBX" % moo2_dir, "2b7d672d890d936072f9bd9c9e9e3d51"),
            'MAINMENU.LBX':	lbx.Archive("%s/MAINMENU.LBX" % moo2_dir, "d6d537ea650a324738ec5adf2dadc529"),
            'OFFICER.LBX':      lbx.Archive("%s/OFFICER.LBX" % moo2_dir, "65cc1180ca1246667bbdaf49cb47517f"),
            'PLANETS.LBX':      lbx.Archive("%s/PLANETS.LBX" % moo2_dir, "6996b43ea2e8329e14e1ad366a804129"),
            'PLNTSUM.LBX':      lbx.Archive("%s/PLNTSUM.LBX" % moo2_dir, "37465fb75f1864fa750f3163acfe6261"),
            'RACEICON.LBX':	lbx.Archive("%s/RACEICON.LBX" % moo2_dir, "db068efad10498daeb38c48b5425f681"),
            'SR_R9_SC.LBX':     lbx.Archive("%s/SR_R9_SC.LBX" % moo2_dir, "07e1d2361881c5373c71ee4a29a0a267"),
            'TECHSEL.LBX':      lbx.Archive("%s/TECHSEL.LBX" % moo2_dir, "db472e2a7f8185536a21db6e9c73a7a5")
	}
    # /load_lbx_archives

    def check_lbx_archives(self):
        check = True
        for arch_key, arch_obj in self.__lbx.items():
            if arch_obj.check_md5():
                print("%s ... MD5 checksum OK" % arch_obj.get_filename())
            else:
                print("ERROR: %s ... md5 checksum FAILED" % arch_obj.get_filename())
                print("    expected MD5: %s" % arch_obj.md5_hexdigest_expected())
                print("      actual MD5: %s" % arch_obj.md5_hexdigest())
#                print("    > possibly corrupted file or not original MOO2 version 1.31 file")
                print("")
                check = False
        return check
#        if self.__md5_hexdigest == md5sum:
#            print("%s md5 checksum OK" % filename)
#        else:
#            print("%s md5 checksum FAILED ... expected %s got %s" % (filename, self.__md5_hexdigest, md5sum))


    def get_lbx_archives(self):
	return self.__lbx
    # /get_lbx_archives

    def load_raw_palettes(self):
	self.__raw_palettes = {
            'FONTS_01': self.__lbx['FONTS.LBX'].read_palette(1),
            'FONTS_02': self.__lbx['FONTS.LBX'].read_palette(2),
            'FONTS_03': self.__lbx['FONTS.LBX'].read_palette(3),
            'FONTS_04': self.__lbx['FONTS.LBX'].read_palette(4),
            'FONTS_05': self.__lbx['FONTS.LBX'].read_palette(5),
            'FONTS_06': self.__lbx['FONTS.LBX'].read_palette(6),
            'FONTS_07': self.__lbx['FONTS.LBX'].read_palette(7),
            'FONTS_08': self.__lbx['FONTS.LBX'].read_palette(8),
            'FONTS_09': self.__lbx['FONTS.LBX'].read_palette(9),
            'FONTS_10': self.__lbx['FONTS.LBX'].read_palette(10),
            'FONTS_11': self.__lbx['FONTS.LBX'].read_palette(11),
            'FONTS_12': self.__lbx['FONTS.LBX'].read_palette(12),
            'FONTS_13': self.__lbx['FONTS.LBX'].read_palette(13),

            'IFONTS_1': self.__lbx['IFONTS.LBX'].read_palette(1),
            'IFONTS_2': self.__lbx['IFONTS.LBX'].read_palette(2),
            'IFONTS_3': self.__lbx['IFONTS.LBX'].read_palette(3),
            'IFONTS_4': self.__lbx['IFONTS.LBX'].read_palette(4)
	}
    # /load_raw_palettes

    def get_raw_palettes(self):
	return self.__raw_palettes
    # /get_raw_palettes
    
    def init_palettes(self):
	self.__palettes = {
            'APP_PICS.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'BUFFER0.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'COLBLDG.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'COLONY2.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'COLPUPS.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'COLSUM.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'COLSYSDI.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'GAME.LBX':         copy.deepcopy(self.__raw_palettes['FONTS_01']),
            'INFO.LBX':         copy.deepcopy(self.__raw_palettes['FONTS_01']),
            'MAINMENU.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_06']),
            'OFFICER.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'PLANETS.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'RACEICON.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02']),
            'SR_R9_SC.LBX':	copy.deepcopy(self.__raw_palettes['FONTS_02'])
	}
    # /init_palettes
    
    def set_palette_color(self, palette_key, color_num, color_def):
	self.__palettes[palette_key][color_num] = color_def
    # /set_palette_color
    
    def get_palettes(self):
	return self.__palettes
    # /get_palettes

    def load_image(self, lbx_key, picture_id, picture_frame, palette_key, color_key):
	return self.__lbx[lbx_key].get_surface(picture_id, picture_frame, self.__palettes[palette_key], color_key)
    # /load_image

    def set_image(self, image_key, image_data):
	self.__images[image_key] = image_data
    # /set_image

    def get_image(self, image_key):
	return self.__images[image_key]
    # /get_image

    def draw_image(self, image_key, target, x, y):
	target.blit(self.get_image(image_key), (x, y))
    # /draw_image

    def draw_screen(self, image_key, target):
	target.blit(self.get_image(image_key), (0, 0))
    # /draw_screen

    def load_splashscreen(self):
	self.__images['MAIN_MENU_BG'] = self.load_image('MAINMENU.LBX', 0, 0, 'MAINMENU.LBX', None)
    # /load_splashscreen

    def draw_splashscreen(self, target):
	self.draw_image('MAIN_MENU_BG', target, 0, 0)
    # /draw_splashscreen

    def load_planet_background(self, picture_num):
	self.__images['PLANET_BACKGROUND'][picture_num] = self.load_image('PLANETS.LBX', picture_num, 0, 'PLANETS.LBX', None)

    def load_graphic(self):
	IMAGES = self.__images

	IMAGES['MAIN_SCREEN'] = {}
	IMAGES['MAIN_SCREEN']['background'] = self.load_image('COLONY2.LBX', 49, 0, 'COLONY2.LBX', None)
	IMAGES['MAIN_SCREEN']['screen'] = self.load_image('BUFFER0.LBX', 0, 0, 'BUFFER0.LBX', 0x00)
	IMAGES['LEADERS_SCREEN'] = self.load_image('OFFICER.LBX', 0, 0, 'BUFFER0.LBX', None)
	IMAGES['MAIN_LEFT_3_BUTTONS'] = self.load_image('BUFFER0.LBX', 3, 0, 'BUFFER0.LBX', None)
	IMAGES['MAIN_ZOOM_IN'] = self.load_image('BUFFER0.LBX', 6, 0, 'BUFFER0.LBX', None)
	IMAGES['MAIN_ZOOM_OUT'] = self.load_image('BUFFER0.LBX', 7, 0, 'BUFFER0.LBX', None)
	IMAGES['MAIN_TURN_ON'] = self.load_image('BUFFER0.LBX', 12, 0, 'BUFFER0.LBX', None)

	#	star map
	IMAGES['MAP_STARS'] = {}
	for star_class in range(7):
	    IMAGES['MAP_STARS'][star_class] = {}
	    for star_size in range(6):
		pic = 148 + (6 * star_class) + (5 - star_size)
		IMAGES['MAP_STARS'][star_class][star_size] = self.load_image('BUFFER0.LBX', pic, 0, 'BUFFER0.LBX', None)

	IMAGES['STARSYSTEM_DIALOG'] = self.load_image('BUFFER0.LBX', 73, 0, 'BUFFER0.LBX', 0x00)

	IMAGES['SYSTEM_ORBIT'] = {}
	for i in range(5):
            IMAGES['SYSTEM_ORBIT'][i] = self.load_image('BUFFER0.LBX', 77 + i, 0, 'BUFFER0.LBX', 0x00)

	IMAGES['SYSTEM_ASTEROIDS'] = {}
	for i in range(1):
	    IMAGES['SYSTEM_ASTEROIDS'][i] = self.load_image('BUFFER0.LBX', 91 + i, 0, 'BUFFER0.LBX', lbx.COLORKEY)

	IMAGES['SYSTEM_STAR'] = {}
	for i in range(6):
	    IMAGES['SYSTEM_STAR'][i] = self.load_image('BUFFER0.LBX', 83 + i, 0, 'BUFFER0.LBX', 0x00)
	IMAGES['SYSTEM_STAR'][6] = self.load_image('BUFFER0.LBX', 184 , 0, 'BUFFER0.LBX', 0x00)

	IMAGES['SYSTEM_PLANETS'] = {}
	for terrain in range(10):
	    IMAGES['SYSTEM_PLANETS'][terrain] = {}
	    for size in range(5):
		IMAGES['SYSTEM_PLANETS'][terrain][size] = self.load_image('BUFFER0.LBX', (92 + (5 * terrain) + size), 0, 'BUFFER0.LBX', None)

	IMAGES['SYSTEM_GAS_GIANTS'] = {}
	for size in range(5):
#	    IMAGES['SYSTEM_GAS_GIANTS'][size] = self.load_image('BUFFER0.LBX', 142 + size, 0, 'BUFFER0.LBX', None)
	    IMAGES['SYSTEM_GAS_GIANTS'][size] = self.load_image('BUFFER0.LBX', 142, 0, 'BUFFER0.LBX', None)

	IMAGES['SYSTEM_COLONY_MARKS'] = {}
	IMAGES['SYSTEM_OUTPOST_MARKS'] = {}
	for color in range(8):
	    IMAGES['SYSTEM_COLONY_MARKS'][color] = self.load_image('BUFFER0.LBX', 35 + color, 0, 'BUFFER0.LBX', 0x00)
	    IMAGES['SYSTEM_OUTPOST_MARKS'][color] = self.load_image('BUFFER0.LBX', 43 + color, 0, 'BUFFER0.LBX', 0x00)

    #	COLONY VIEW

	# planets backgrounds are lazy loaded...
	IMAGES['PLANET_BACKGROUND'] = {}
	for i in range(30):
	    IMAGES['PLANET_BACKGROUND'][i] = None

	IMAGES['COLONY_SCREEN'] = {}
	IMAGES['COLONY_SCREEN']['background'] = IMAGES['MAIN_SCREEN']['background']

	IMAGES['COLONY_SCREEN']['panel'] = self.load_image('COLPUPS.LBX', 5, 0, 'COLPUPS.LBX', None)
	IMAGES['COLONY_SCREEN']['planet_arrow'] = self.load_image('COLSYSDI.LBX', 64, 0, 'COLSYSDI.LBX', 0x00)

	IMAGES['COLONY_SCREEN']['planet_schemes'] = {}
	for terrain in range(10):
#	    IMAGES['COLONY_SCREEN']['planet_schemes'].append([])
	    IMAGES['COLONY_SCREEN']['planet_schemes'][terrain] = {}
	    for size in range(5):
#		IMAGES['COLONY_SCREEN']['planet_schemes'][terrain].append(LBX['COLSYSDI.LBX', 11 + (5 * terrain) + size, 0, 'COLSYSDI.LBX']))
		IMAGES['COLONY_SCREEN']['planet_schemes'][terrain][size] = self.load_image('COLSYSDI.LBX', 11 + (5 * terrain) + size, 0, 'COLSYSDI.LBX', None)

	IMAGES['COLONY_SCREEN']['gasgiant_scheme'] = self.load_image('COLSYSDI.LBX', 61, 0, 'COLSYSDI.LBX', 0x00)
	IMAGES['COLONY_SCREEN']['asteroids_scheme'] = self.load_image('COLSYSDI.LBX', 63, 0, 'COLSYSDI.LBX', 0x00)

	IMAGES['RACEICON.LBX'] = {}
	for i in range(13):
#	    IMAGES['RACEICON.LBX'].append([])
	    IMAGES['RACEICON.LBX'][i] = {}
	    for ii in range(13):
#		IMAGES['RACEICON.LBX'][i].append(LBX['RACEICON.LBX', (i * 13) + ii, 0, 'RACEICON.LBX'], 0x00))
		IMAGES['RACEICON.LBX'][i][ii] = self.load_image('RACEICON.LBX', (i * 13) + ii, 0, 'RACEICON.LBX', 0x00)

	IMAGES['COLONY2.LBX'] = {}

	IMAGES['COLONY2.LBX']['1food'] = self.load_image('COLONY2.LBX', 0, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['1production'] = self.load_image('COLONY2.LBX', 1, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['1research'] = self.load_image('COLONY2.LBX', 2, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['1money'] = self.load_image('COLONY2.LBX', 3, 0, 'COLONY2.LBX', 0x00)

	IMAGES['COLONY2.LBX']['10food'] = self.load_image('COLONY2.LBX', 4, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['10production'] = self.load_image('COLONY2.LBX', 5, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['10research'] = self.load_image('COLONY2.LBX', 6, 0, 'COLONY2.LBX', 0x00)
	IMAGES['COLONY2.LBX']['10money'] = self.load_image('COLONY2.LBX', 3, 0, 'COLONY2.LBX', 0x00)

	IMAGES['OFFICER.LBX'] = {}
	IMAGES['OFFICER.LBX']['buttons'] = {}
	IMAGES['OFFICER.LBX']['buttons']['colony_leaders'] = self.load_image('OFFICER.LBX', 3, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['buttons']['ship_officers'] = self.load_image('OFFICER.LBX', 4, 0, 'BUFFER0.LBX', None)

	IMAGES['OFFICER.LBX']['leaders'] = {}
	for i in range(67):
	    IMAGES['OFFICER.LBX']['leaders'][i] = self.load_image('OFFICER.LBX', 21 + i, 0, 'BUFFER0.LBX', None)

	IMAGES['OFFICER.LBX']['skill_icons'] = {}
	IMAGES['OFFICER.LBX']['skill_icons']['assassin'] = self.load_image('OFFICER.LBX', 88, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['commando'] = self.load_image('OFFICER.LBX', 89, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['diplomat'] = self.load_image('OFFICER.LBX', 90, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['91'] = self.load_image('OFFICER.LBX', 91, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['92'] = self.load_image('OFFICER.LBX', 92, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['famous'] = self.load_image('OFFICER.LBX', 93, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['farmer'] = self.load_image('OFFICER.LBX', 94, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['fighter_pilot'] = self.load_image('OFFICER.LBX', 95, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['financial_leader'] = self.load_image('OFFICER.LBX', 96, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['galactic_role'] = self.load_image('OFFICER.LBX', 97, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['helmsman'] = self.load_image('OFFICER.LBX', 98, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['instructor'] = self.load_image('OFFICER.LBX', 99, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['labor_leader'] = self.load_image('OFFICER.LBX', 100, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['medicine'] = self.load_image('OFFICER.LBX', 101, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['megawealth'] = self.load_image('OFFICER.LBX', 102, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['navigator'] = self.load_image('OFFICER.LBX', 103, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['operations'] = self.load_image('OFFICER.LBX', 104, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['ordnance'] = self.load_image('OFFICER.LBX', 105, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['researcher'] = self.load_image('OFFICER.LBX', 106, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['science_leader'] = self.load_image('OFFICER.LBX', 107, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['108'] = self.load_image('OFFICER.LBX', 108, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['spiritual_leader'] = self.load_image('OFFICER.LBX', 109, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['spy_master'] = self.load_image('OFFICER.LBX', 110, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['tactics'] = self.load_image('OFFICER.LBX', 111, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['telepath'] = self.load_image('OFFICER.LBX', 112, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['trader'] = self.load_image('OFFICER.LBX', 113, 0, 'BUFFER0.LBX', None)
	IMAGES['OFFICER.LBX']['skill_icons']['weaponry'] = self.load_image('OFFICER.LBX', 114, 0, 'BUFFER0.LBX', None)

	IMAGES['MAP_SHIPS'] = {}
	for c in range(9):
	    IMAGES['MAP_SHIPS'][c] = {}
	    for i in range(4):
		IMAGES['MAP_SHIPS'][c][i] = self.load_image('BUFFER0.LBX', 205 + (c * 4 + i), 0, 'BUFFER0.LBX', None)

	for c in range(9, 14):
	    IMAGES['MAP_SHIPS'][c] = {}
	    for i in range(4):
		IMAGES['MAP_SHIPS'][c][i] = self.load_image('BUFFER0.LBX', 205 + (c * 4 + (3 - i)), 0, 'BUFFER0.LBX', None)

	IMAGES['GOVERNMENT_ICONS'] = {}
	for i in range(8):
	    IMAGES['GOVERNMENT_ICONS'][i] = self.load_image('COLONY2.LBX', 19 + i, 0, 'COLONY2.LBX', 0x00)

	IMAGES['MORALE_ICONS'] = {}
	IMAGES['MORALE_ICONS']['good'] = self.load_image('COLONY2.LBX', 16, 0, 'COLONY2.LBX', 0x00)
	IMAGES['MORALE_ICONS']['bad'] = self.load_image('COLONY2.LBX', 17, 0, 'COLONY2.LBX', 0x00)

	IMAGES['GAME_MENU'] = {}
	IMAGES['GAME_MENU']['main'] = self.load_image('GAME.LBX', 0, 0, 'GAME.LBX', None)

	IMAGES['GAME_MENU']['settings'] = self.load_image('GAME.LBX', 29, 0, 'GAME.LBX', None)
	IMAGES['GAME_MENU']['settings1'] = self.load_image('GAME.LBX', 31, 0, 'GAME.LBX', None)
	IMAGES['GAME_MENU']['settings2'] = self.load_image('GAME.LBX', 8, 0, 'GAME.LBX', None)
	IMAGES['GAME_MENU']['settings3'] = self.load_image('GAME.LBX', 30, 0, 'GAME.LBX', None)

	IMAGES['GAME_MENU']['load_game'] = self.load_image('GAME.LBX', 11, 0, 'GAME.LBX', None)
	IMAGES['GAME_MENU']['save_game'] = self.load_image('GAME.LBX', 14, 0, 'GAME.LBX', None)

	# main palette debug
#	i = 0
#	for color in PALETTES['BUFFER0.LBX']:
#	    print "%i ... %s" % (i, color)
#	    i += 1

#	PALETTES['BUFFER0.LBX'][32] = {'alpha': 1, 'rgb': 0xECECEC}	# 000c10 -> ececec
#	PALETTES['BUFFER0.LBX'][33] = {'alpha': 1, 'rgb': 0xF4F4F4}	# 00141c -> f4f4f4
#	PALETTES['BUFFER0.LBX'][34] = {'alpha': 1, 'rgb': 0xFCFCFC}	# 0c2834 -> fcfcfc
#	PALETTES['BUFFER0.LBX'][45] = {'alpha': 1, 'rgb': 0xAC9C94}	# 200c10 -> ac9c94
#	PALETTES['BUFFER0.LBX'][108] = {'alpha': 1, 'rgb': 0x000818}	# f4e094 -> 000818
#	PALETTES['BUFFER0.LBX'][109] = {'alpha': 1, 'rgb': 0x000818}	# fcecb0 -> 000818

	self.set_palette_color('BUFFER0.LBX', 32, {'alpha': 1, 'rgb': 0xECECEC})	# 000c10 -> ececec
	self.set_palette_color('BUFFER0.LBX', 33, {'alpha': 1, 'rgb': 0xF4F4F4})	# 00141c -> f4f4f4
	self.set_palette_color('BUFFER0.LBX', 34, {'alpha': 1, 'rgb': 0xFCFCFC})	# 0c2834 -> fcfcfc
	self.set_palette_color('BUFFER0.LBX', 45, {'alpha': 1, 'rgb': 0xECECEC})	# 200c10 -> ac9c94
	self.set_palette_color('BUFFER0.LBX', 108, {'alpha': 1, 'rgb': 0xECECEC})	# f4e094 -> 000818
	self.set_palette_color('BUFFER0.LBX', 109, {'alpha': 1, 'rgb': 0xECECEC})	# fcecb0 -> 000818

	IMAGES['INFO_SCREEN'] = {}
	IMAGES['INFO_SCREEN']['panel'] = self.load_image('INFO.LBX', 0, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['return'] = self.load_image('INFO.LBX', 2, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['history_graph_off'] = self.load_image('INFO.LBX', 3, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['tech_review_off'] = self.load_image('INFO.LBX', 4, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['race_statistics_off'] = self.load_image('INFO.LBX', 5, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['turn_summary_off'] = self.load_image('INFO.LBX', 6, 0, 'BUFFER0.LBX', None)
	IMAGES['INFO_SCREEN']['reference_off'] = self.load_image('INFO.LBX', 7, 0, 'BUFFER0.LBX', None)

	IMAGES['APP_PICS'] = {}
#	PALETTES['APP_PICS.LBX'][176] = {'alpha': 1, 'rgb': 0x000c00}	# 686034 -> 000c00
#	PALETTES['APP_PICS.LBX'][177] = {'alpha': 1, 'rgb': 0x081c08}	# 787040 -> 081c08
#	PALETTES['APP_PICS.LBX'][179] = {'alpha': 1, 'rgb': 0x084408}	# b4ac78 -> 084408
#	PALETTES['APP_PICS.LBX'][180] = {'alpha': 1, 'rgb': 0x084408}	# b4ac78 -> 084408
#	PALETTES['APP_PICS.LBX'][181] = {'alpha': 1, 'rgb': 0x085008}	# c8c48c -> 085008

	self.set_palette_color('BUFFER0.LBX', 176, {'alpha': 1, 'rgb': 0x000c00})	# 686034 -> 000c00
	self.set_palette_color('BUFFER0.LBX', 177, {'alpha': 1, 'rgb': 0x081c08})	# 787040 -> 081c08
	self.set_palette_color('BUFFER0.LBX', 179, {'alpha': 1, 'rgb': 0x084408})	# b4ac78 -> 084408
	self.set_palette_color('BUFFER0.LBX', 180, {'alpha': 1, 'rgb': 0x084408})	# b4ac78 -> 084408
	self.set_palette_color('BUFFER0.LBX', 181, {'alpha': 1, 'rgb': 0x085008})	# c8c48c -> 085008

	IMAGES['RESEARCH_DIALOG'] = {}
	IMAGES['RESEARCH_DIALOG']['panel'] = self.load_image('TECHSEL.LBX', 0, 0, 'BUFFER0.LBX', None)

	IMAGES['COLONY_BUILD_SCREEN'] = {}
	IMAGES['COLONY_BUILD_SCREEN']['screen'] = self.load_image('COLBLDG.LBX', 0, 0, 'COLBLDG.LBX', None)

	IMAGES['PLANETS_SCREEN'] = {}
	IMAGES['PLANETS_SCREEN']['screen'] = self.load_image('PLNTSUM.LBX', 0, 0, 'BUFFER0.LBX', None)

	for i in range(212):
	    if i == 0:
		IMAGES['APP_PICS'][i] = self.load_image('APP_PICS.LBX', i, 0, 'APP_PICS.LBX', None)
	    elif i == 155:
		IMAGES['APP_PICS'][i] = self.load_image('APP_PICS.LBX', i, 0, 'APP_PICS.LBX', None)
	    else:
		IMAGES['APP_PICS'][i] = None

    # /load_graphic

    def get_images(self):
	return self.__images
    # /get_images

    def bordered_text(self, surface, text, color, border, x, y, font):
        surface.blit(font.render(text, 1, border), (x - 1, y))
        surface.blit(font.render(text, 1, border), (x + 1, y))
        surface.blit(font.render(text, 1, border), (x, y - 1))
        surface.blit(font.render(text, 1, border), (x, y + 1))
        surface.blit(font.render(text, 1, color), (x, y))



import math

from _buildings import *

from _game_constants import *
import _research
import _tech_table
import _governments

__author__="peterman"
__date__ ="$Jan 2, 2010 12:42:33 PM$"

PLANETS_MINERALS = {
    0: { 'worker_base':	1 },
    1: { 'worker_base':	2 },
    2: { 'worker_base':	3 },
    3: { 'worker_base':	5 },
    4: { 'worker_base':	8 },
}

PLANETS_SPECIALS = {
    SPECIAL_ARTIFACTS: {
	'research_bonus':		+2,
    }
}

DEFAULT_RULES = {

    'hero_levels':                      (0, 60, 150, 300, 1000, 2000),
    'planets_minerals':			PLANETS_MINERALS,
    'planets_specials':			PLANETS_SPECIALS,
    'research':				_research.RESEARCH,
    'research_areas':			_research.RESEARCH_AREAS,
    'tech_table':			_tech_table.TECH_TABLE,
    'buildings':			BUILDINGS,
    'governments':                      _governments.GOVERNMENTS,
}

"""
    'planet_worker_base':                           [1, 2, 3, 5, 8],
    'microlite_construction_per_worker':            1,
    'microlite_construction_industry_bonus':        1,

#    'automated_factory':                            5,
#    'automated_factory_per_worker':                 1,
#    'automated_factory_industry_bonus':             1,

#    'robo_miner_plant':                             10,
#    'robo_miner_plant_per_worker':                  2,
#    'robo_miner_plant_industry_bonus':              2,

#    'deep_core_mine':                               15,
#    'deep_core_mine_per_worker':                    3,
#    'deep_core_mine_industry_bonus':                3,

    'robotic_factory_mineral_bonuses':              [5, 8, 10, 15, 20],

    'weather_controller_farmer_bonus':              2,

    'astro_university_farmer_bonus':                1,
    'astro_university_worker_bonus':                1,
    'astro_university_scientist_bonus':             1,

    'research_lab':                                 5,
    'research_lab_scientist_bonus':                 1,
    'supercomputer':                                10,
    'supercomputer_scientist_bonus':                2,

    'autolab':                                      30,


    'heightened_intelligence_research_bonus':       1,
    'artifacts_reseach_bonus':                      2,
"""

"""
    returns the Food production summary table
"""
def compose_food_summary(RULES, colony, PLAYERS):
    planet = colony.planet()

#    if planet is None:
#        print("ERROR: colony has no planet!")
#        print("colony.owner() = %i" % colony.get_owner())

#    print("rules::compose_food_summary() colony.get_id() = %i" % colony.get_id())
#    print("rules::compose_food_summary() planet.get_id() = %i" % planet.get_id())

    summary = {
        'farmers':			0,
#        'astro_university':		0,
        'aquatic_bonus':		0,
#        'weather_controller':	0,
#        'morale_bonus':		0,
#        'hydroponic_farm':		0,
#        'blockaded':		0,
#        'government_bonus':		0
    }

    RULES_BUILDINGS	= RULES['buildings']

    farmers_num = len(colony.colonists[FARMER])

    goverment = PLAYERS[colony.get_owner()].get_racepick_item('goverment')

    for colonist in colony.colonists[FARMER]:
        summary['farmers'] += planet.get_foodbase()
        if PLAYERS[colonist['race']].get_racepick_item('aquatic'):
#		print "	aquatic farmer!"
            if planet.get_terrain() in [TERRAIN_OCEAN, TERRAIN_TERRAN]:
                summary['aquatic_bonus'] += 1

    # apply buildings farmer_bonus
    for b_id in colony.list_buildings():
	if BUILDINGS[b_id].has_key("farmer_bonus"):
#	    b_sum_key = "building_%i" % b_id
	    b_sum_key = "%s_bonus" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = farmers_num * RULES_BUILDINGS[b_id]['farmer_bonus']

    # count in morale
    summary['morale_bonus'] = float(sum(summary.values())) * colony.morale() / 100

    # apply buildings food_production
    for b_id in colony.list_buildings():
	if RULES_BUILDINGS[b_id].has_key("food_production"):
#	    b_sum_key = "building_%i" % b_id
	    b_sum_key = "%s" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = RULES_BUILDINGS[b_id]['food_production']

    # TODO: apply blockade
    # TODO: apply food replicators

    if goverment == GOVERMENT_UNIFICATION:
        summary['government_bonus'] = sum(summary.values()) // 2

    return summary
# /compose_food_summary

"""
    returns the hashmap of production items
"""
def compose_industry_summary(RULES, colony, colony_leader, PLAYERS):
    BUILDINGS = RULES['buildings']

    TECH_TABLE = RULES['tech_table']

    ME = PLAYERS[colony.get_owner()]

    planet = colony.planet()
    summary = {
        'workers':			0,
#        'microlite_constructions':	0,
#        'automated_factory_people':	0,
#        'deep_core_mine_people':	0,
#        'robominer_plant_people':	0,
#        'morale_bonus':                 0,
        'gravity_penalty':		0,
#        'blockaded':                    0,
#        'government_bonus':		0,
#        'recyclotron':                  0,
#        'automated_factory_bonus':	0,
#        'robominer_plant_bonus':	0,
#        'deep_core_mine_bonus':         0,
#        'robotic_factory':		0,
        'pollution':                    0,
    }

    planet_minerals = planet.get_minerals()

    worker_base = RULES['planets_minerals'][planet_minerals]['worker_base']
    player_goverment = PLAYERS[colony.get_owner()].get_racepick_item('goverment')
    industry_per_worker = worker_base

    workers_num = len(colony.colonists[WORKER])

    for colonist in colony.colonists[WORKER]:
        summary['workers'] += worker_base + PLAYERS[colonist['race']].get_racepick_item('industry')


    # TODO: appply technologies worker_bonus
    for tech_id in ME.get_known_techs():
        if TECH_TABLE[tech_id].has_key("worker_bonus"):
	    b_sum_key = "%s ... bonus" % TECH_TABLE[tech_id]['name']
            summary[b_sum_key] = workers_num * TECH_TABLE[tech_id]['worker_bonus']
            industry_per_worker += TECH_TABLE[tech_id]['worker_bonus']

    # appply buildings worker_bonus
    for b_id in colony.list_buildings():
	if BUILDINGS[b_id].has_key("worker_bonus"):
	    b_sum_key = "%s ... bonus" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = workers_num * BUILDINGS[b_id]['worker_bonus']
	    industry_per_worker += BUILDINGS[b_id]['worker_bonus']

    tmp_sum = float(sum(summary.values()))
#    print "rules::compose_industry_summary() ... tmp_sum = %f" % tmp_sum


    # TODO: apply morale!
    morale_bonus = tmp_sum * colony.morale() / 100

    if colony_leader and colony_leader['skills'].has_key('industry_bonus'):
#        print colony_leader
        leader_bonus = colony_leader['skills']['industry_bonus'] + (colony_leader['level'] * colony_leader['skills']['industry_bonus'])
#        print "rules::compose_industry_summary() ... leader_bonus = %f" % leader_bonus
        leader_bonus = tmp_sum * leader_bonus / 100
#        print "rules::compose_industry_summary() ... leader_bonus = %f" % leader_bonus
#                   print "leader_bonus = %i" % leader_bonus
#        summary['Leader Bonus'] = round((colony_leader['skills']['morale_bonus'] + (colony_leader['level'] * colony_leader['skills']['morale_bonus'])) / 10) * 10
    else:
        leader_bonus = 0



    # TODO: apply government
    if player_goverment == 6:			# GOVERMENT_UNIFICATION
        summary['government_bonus'] = tmp_sum * 0.5

    if morale_bonus:
        summary['Moral Bonus'] = morale_bonus
    if leader_bonus:
        summary['Leader Bonus'] = leader_bonus


#	print "industry_per_worker: %i" % industry_per_worker
#	print "planet gravity: %i" % planet['gravity']
    if not colony.has_building(B_GRAVITY_GENERATOR):
        for colonist in colony.colonists[WORKER]:
            if PLAYERS[colonist['race']].get_racepick_item('low_g'):
                # log-G race
                if planet.get_gravity() == 1:				# normal-G planet
                    summary['gravity_penalty'] -= float((industry_per_worker + PLAYERS[colonist['race']].get_racepick_item('industry'))) / 4
            elif PLAYERS[colonist['race']].get_racepick_item('high_g'):
                pass
            else:
                # normal-G race
                if planet.get_gravity() == 2:				# high-G planet
                    summary['gravity_penalty'] -= float((industry_per_worker + PLAYERS[colonist['race']].get_racepick_item('industry'))) / 2


    total = round(sum(summary.values()))
#	print "@@@ colony::get_industry_summary ... total = %s" % str(total)

#    summary['pollution'] -= colony.get_colony_pollution(total, PLAYERS)
    summary['pollution'] -= count_colony_pollution(colony, total, PLAYERS)

    if colony.has_building(B_RECYCLOTRON):
        summary['recyclotron'] = len(colony.colonists[FARMER]) + workers_num + len(colony.colonists[SCIENTIST])

    # apply buildings industry
    for b_id in colony.list_buildings():
	if BUILDINGS[b_id].has_key("industry"):
	    b_sum_key = "%s ... industry" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = BUILDINGS[b_id]['industry']

    # apply buildings industry_by_minerals
    for b_id in colony.list_buildings():
	if BUILDINGS[b_id].has_key("industry_by_minerals"):
	    b_sum_key = "building_%s_minerals" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = BUILDINGS[b_id]['industry_by_minerals'][planet_minerals]


    # TODO: apply food replicator: original ratio is 2 production units -> 1 food

    return summary
# /compose_industry_summary

"""
    returns the hasmap of research items
"""
def compose_research_summary(RULES, colony, PLAYERS):
    planet = colony.planet()
    summary = {
        'scientists':		0,
#        'artifacts':		0,
#        'intelligence':		0,
#        'supercomputer_people':	0,
#        'supercomputer_bonus':	0,
#        'research_lab_people':	0,
#        'research_lab_bonus':	0,
        'gravity_penalty':		0.0,
#        'astro_university':		0,
        'race_bonus':		0,
#        'morale_bonus':		0,
#        'government_bonus':		0,
#        'autolab':			0
    }

    if colony.is_outpost():
        return summary
    base = 3

    player_goverment = PLAYERS[colony.get_owner()].get_racepick_item('goverment')

    scientist_num = len(colony.colonists[SCIENTIST])

    morale = float(colony.morale())

    for colonist in colony.colonists[SCIENTIST]:
        race = colonist['race']
        summary['scientists'] += base
        summary['race_bonus'] += PLAYERS[race].get_racepick_item('science')

#        if PLAYERS[race]['technologies'][TECH_HEIGHTENED_INTELLIGENCE] == 3:
        if TECH_HEIGHTENED_INTELLIGENCE in PLAYERS[race].get_known_techs():
            summary['intelligence'] += RULES['heightened_intelligence_research_bonus']

	if RULES['planets_specials'].has_key(planet.get_special()):
	    if RULES['planets_specials'][planet.get_special()].has_key("research_bonus"):
		summary['special'] = RULES['planets_specials'][planet.get_special()]['research_bonus']

#        if planet['special'] == SPECIAL_ARTIFACTS:
#            research['artifacts'] += RULES['artifacts_reseach_bonus']

        if not colony.has_building(B_GRAVITY_GENERATOR):
            if PLAYERS[race].get_racepick_item('low_g'):
                # low-G colonist
                if planet.get_gravity() == PLANET_NORMAL_G:	# normal-G planet
                    summary['gravity_penalty'] -= 1.75
            elif PLAYERS[race].get_racepick_item('high_g'):
                # high-G colonist
                pass
            else:
                # normal-G colonist
                if planet.get_gravity() == PLANET_HEAVY_G:
                    summary['gravity_penalty'] -= float(base) / 2

    # appply buildings scientist_bonus
    if scientist_num:
        for b_id in colony.list_buildings():
            if BUILDINGS[b_id].has_key("scientist_bonus"):
                b_sum_key = "%s ... bonus" % BUILDINGS[b_id]['name']
                summary[b_sum_key] = scientist_num * BUILDINGS[b_id]['scientist_bonus']

    # Government bonus
    morale_bonus = (morale * (float(sum(summary.values())))) / 100

    if player_goverment == GOVERMENT_DEMOCRACY:
        summary['government_bonus'] = float(sum(summary.values()) - summary['gravity_penalty']) * 0.5
    elif player_goverment == GOVERMENT_FEUDAL:
        summary['government_bonus'] = round(round(sum(summary.values()) - summary['gravity_penalty']) * -0.5)

    if morale_bonus:
        summary['morale_bonus'] = morale_bonus

    # appply buildings research
    for b_id in colony.list_buildings():
	if BUILDINGS[b_id].has_key("research"):
	    b_sum_key = "%s ... research" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = BUILDINGS[b_id]['research']

    return summary
# /compose_research_summary

def compose_morale_summary(RULES, colony, colony_leader, PLAYERS):
    ME = PLAYERS[colony.get_owner()]
    BUILDINGS = RULES['buildings']
    TECH_TABLE = RULES['tech_table']

    summary = {
#        'government_morale':	0,
#        'marine_barracks':		0,
#        'psionics':			0,
#        'holo_simulator':		0,
#        'virtual_reality_network':	0
    }

#	print(colony.get_owner())
#	print(PLAYERS[colony.get_owner()])
#	print(PLAYERS[colony.get_owner()]['racepicks'])
    player_goverment = PLAYERS[colony.get_owner()].get_racepick_item('goverment')
    summary['Government Morale'] = RULES['governments'][player_goverment]['morale']
#	print "@@@ colony::get_morale_summary ... goverment = %i" % player_goverment

    # Marine Barracks ... Eliminates morale penalties for dictatorship and feudal governments
#    if (colony.buildings[B_MARINE_BARRACKS]) and (player_goverment in [GOVERMENT_FEUDAL, GOVERMENT_FEUDAL2, GOVERMENT_DICTATORSHIP, GOVERMENT_IMPERIUM]):
#    if colony.has_building(B_MARINE_BARRACKS) and (player_goverment in [GOVERMENT_FEUDAL, GOVERMENT_FEUDAL2, GOVERMENT_DICTATORSHIP, GOVERMENT_IMPERIUM]):
#        summary['marine_barracks'] = 20

    # Holo Simulator ... Increases a planet's morale by +20%
#    if colony.has_building(B_HOLO_SIMULATOR):
 #       summary['holo_simulator'] = 20

    
    for b_id in colony.list_buildings():

        # appply buildings government_morale_bonus
	if BUILDINGS[b_id].has_key("government_morale_bonus"):
	    b_sum_key = "%s_gov" % BUILDINGS[b_id]['name']
	    summary[b_sum_key] = BUILDINGS[b_id]['government_morale_bonus'][player_goverment]

        # appply buildings morale_bonus
	if BUILDINGS[b_id].has_key("morale_bonus"):
	    b_sum_key = BUILDINGS[b_id]['name']
	    summary[b_sum_key] = BUILDINGS[b_id]['morale_bonus']

    # Psionics ... Morale on all planets is increased by 10% for dictatorship and imperium government

#	print "@@@ colony::get_morale_summary ... TECH_PSIONICS = %i" % PLAYERS[colony.get_owner()]['technologies'][TECH_PSIONICS]
#    if (PLAYERS[colony.get_owner()]['technologies'][TECH_PSIONICS] == 3) and (player_goverment in [GOVERMENT_DICTATORSHIP, GOVERMENT_IMPERIUM]):
#    if (TECH_PSIONICS in PLAYERS[colony.get_owner()].get_known_techs()) and (player_goverment in [GOVERMENT_DICTATORSHIP, GOVERMENT_IMPERIUM]):
#        summary['psionics'] = 10
    
    for tech_id in ME.get_known_techs():

        # appply technologies government_morale_bonus
        if TECH_TABLE[tech_id].has_key("government_morale_bonus"):
	    b_sum_key = "%s_gov" % TECH_TABLE[tech_id]['name']
            summary[b_sum_key] = TECH_TABLE[tech_id]['government_morale_bonus'][player_goverment]

        # appply technologies morale_bonus
        if TECH_TABLE[tech_id].has_key("morale_bonus"):
	    b_sum_key = "%s_bonus" % TECH_TABLE[tech_id]['name']
            summary[b_sum_key] = TECH_TABLE[tech_id]['morale_bonus']

    if colony_leader and colony_leader['skills'].has_key('morale_bonus'):
#        print colony_leader
#        print "leader morale_bonus = %i" % leader_morale_bonus
        summary['Leader Bonus'] = round((colony_leader['skills']['morale_bonus'] + (colony_leader['level'] * colony_leader['skills']['morale_bonus'])) / 10) * 10

    return summary
# /compose_morale_summary

"""
    Returns BC Summary
"""
def compose_bc_summary(RULES, colony, PLAYERS):
    planet = colony.planet()

    summary = {
        'taxes_collected':		0,
        'special_income':		0,
        'morale_bonus':		0,
        'government_bonus':		0,
        'stock_exchange':		0,
        'spaceport':		0,
        'trade_goods':		0
    }

    # Taxes Collected
    # TODO: do natives and androids produce taxes too?
    summary['taxes_collected'] = len(colony.colonists[FARMER] + colony.colonists[WORKER] + colony.colonists[SCIENTIST])

    # Special Income - Gold Deposits
    if planet.get_special() == 4:
        summary['special_income'] = 5

    # Morale Bonus
    morale = float(colony.morale())
    summary['morale_bonus'] = round(morale * (float(sum(summary.values()))) / 100)

    # Government Bonus
    player_goverment = PLAYERS[colony.get_owner()].get_racepick_item('goverment')
    if player_goverment == GOVERMENT_DEMOCRACY:
        summary['government_bonus'] = int(sum(summary.values()) / 2)

    # Planetary Stock Exchange
    if colony.has_building(B_STOCK_EXCHANGE):
        summary['stock_exchange'] = summary['taxes_collected']

    # Spaceport
    if colony.has_building(B_SPACEPORT):
        summary['spaceport'] = int(summary['taxes_collected'] / 2)

    # TODO: Trade Goods ... round the 50% of industry
    build_queue = colony.build_queue()
#	print build_queue[0]
#	print build_queue[1]
#	print build_queue[2]
#	print build_queue[3]
#	print build_queue[4]
#	print build_queue[5]
#	print build_queue[6]
    if build_queue[0]['item'] == BUILD_TRADE_GOODS:
        summary['trade_goods'] = int(colony.get_industry() / 2)


    return summary
# /compose_bc_summary

def get_empty_max_populations():
    return [0, 0, 0, 0, 0, 0, 0, 0]
# /get_empty_max_populations

"""
http://masteroforion2.blogspot.com/2005/10/maximum-population.html
"""
def compose_max_populations(colony, PLAYERS):
    planet = colony.planet()

    max_populations = get_empty_max_populations()
    if colony.is_outpost():
        return max_populations

    size = planet.get_size()
    terrain = planet.get_terrain()

    size_multiplier = [5, 10, 15, 20, 25][size]

    for i in range(8):
        player = PLAYERS[i]
#		print player['racepicks']

        if player.get_racepick_item('aquatic'):
            terrain_multiplier = [25, 25, 25, 25, 80, 100, 80, 60, 100, 100][terrain]	# Aquatic
        else:
            terrain_multiplier = [25, 25, 25, 25, 25, 25, 40, 60, 80, 100][terrain]	# default

        if player.get_racepick_item('tolerant'):
            terrain_multiplier += 25

        terrain_multiplier = min(terrain_multiplier, 100)

        if player.get_racepick_item('subterranean'):
            bonus = [2, 4, 6, 8, 10][size]		# Subterranean = 2 pop per each size class
        else:
            bonus = 0

    # TODO: Advanced City Planning ... bonus + 5

        max_populations[i] = round((float(size_multiplier * terrain_multiplier) / 100) + bonus)

        if colony.has_building(B_BIOSPHERES):
            max_populations[i] += 2

    present = [0, 0, 0, 0, 0, 0, 0, 0]
    for t in [FARMER, WORKER, SCIENTIST]:
        for colonist in colony.colonists[t]:
#	    	print "race: %i" % colonist['race']
            present[colonist['race']] = 1

#    	print "$$$ max_populations: %s $$$ before present filter" % max_populations
    for i in range(8):
        max_populations[i] = int(max_populations[i] * present[i])

    return max_populations
# /get_max_populations

def count_colony_pollution(colony, production, players):
#    print "$$$ count_colony_pollution $$$"
    planet = colony.planet()
    if colony.is_outpost():
        return 0


    if colony.has_building(B_CORE_WASTE_DUMP):
        return 0

    production = float(production)

    tolerant = 0
    pop = 0

    for t in (FARMER, SCIENTIST, WORKER):
        for colonist in colony.colonists[t]:
            pop += 1
            if players[colonist['race']].get_racepick_item('tolerant'):
                tolerant += 1

#	print "    population: %i" % pop
#	print "    tolerant: %i" % tolerant
#	print "    non-tolerant: %i" % (pop - tolerant)
#	print "    planet size: %i" % planet['size']

#	print "	production before atm. renewer: %s" % str(production)
    if colony.has_building(B_ATMOSPHERE_RENEWER):
        production = math.ceil(production / 4)
#	print "	production after atm. renewer: %s" % str(production)

    tolerance = [2, 4, 6, 8, 10][planet.get_size()]

#	if PLAYERS[self.get_owner()]['technologies'][TECH_NANO_DISASSEMBLERS] == 3:
    if TECH_NANO_DISASSEMBLERS in players[colony.get_owner()].get_known_techs():
        tolerance += tolerance

#	print "    planet tolerance: %i" % tolerance
    pollution = float(max(0, production - tolerance)) / 2
#	print "    pollution: %s" % str(pollution)

    pollution = float(pop - tolerant) * pollution / float(pop)
#	print "    pollution: %s" % str(pollution)

    return round(pollution)
# /get_colony_pollution


def count_summary_result(summary):
    sum = 0.0
    for k in summary:
        sum += float(summary[k])
    return round(sum)

def research_costs(research_areas, area, rp):
    if not area:
        return -1
    else:
        return research_areas[area]['cost']
# /research_costs

def research_turns(cost, progress, rp):
    if not rp:
        return -1
    else:
        cost = float(cost)
        progress = float(progress)
        rp = float(rp)
#        print("rules::research_turns() ... cost = %s" % str(cost))
#        print("rules::research_turns() ... rp = %s" % str(rp))
        return int(math.ceil((cost - progress) / rp))
#        return int(math.ceil((cost/rp)+math.sqrt(2*(cost/rp)-1)))
# /research_turns


import math

import lbx

from _buildings import *
from _game_constants import *

import rules
import universe

class Colony(universe.GameObject):

    def __init__(self, colony_id):
	self.__planet = None
        self.set_id(colony_id)

#    def __print_summary_item(self, data, key, title):
#        if data.has_key(key) and data[key]:
#            print "+     %s ... %s    +" % (str(data[key]).ljust(6), title.ljust(25))

    def set_planet_id(self, planet_id):
        self.__planet_id = planet_id

    def get_planet_id(self):
        return self.__planet_id

    def print_info(self):
	print "	owner: %i" % self.get_owner()

    def import_from_moo2(self, data):

	def read_population(data, population):
	    colonists = {0x02: [], 0x03: [], 0x82: []}
	    for i in range(population):
		offset = 0x0C + (4 * i)
		t = (lbx.read_char(data, offset) & 0x80) + (lbx.read_char(data, offset + 1) & 3)
    		colonists[t].append({
		    'a':	lbx.read_char(data, offset),
		    'b':	lbx.read_char(data, offset + 1),
		    'c':	lbx.read_char(data, offset + 2),
		    'd':	lbx.read_char(data, offset + 3),
		    'r1':	(lbx.read_char(data, offset) & 0x70) >> 4,
		    'race':	(lbx.read_char(data, offset) & 0x07)
		})
	    return colonists
	# end func read_population

#	print "Colony::load_from_moo2()"
#	print "	data length: %i" % len(data)
#	self.colony_id			= colony_id
	self.set_owner(lbx.read_char(data,  0x000))
#        print("     Colony::import_from_moo2() ... self.__owner = %i" % self.__owner)
	self.allocated_to	= lbx.read_char(data,  0x001)
        self.set_planet_id(lbx.read_char(data,  0x002))
#        print("     Colony::import_from_moo2() ... self.planet_id = %i" % self.planet_id)
        self.__officer		= lbx.read_short_int(data, 0x004),   # not used ?
        self.__is_outpost	= lbx.read_char(data,  0x006)
#        print("     Colony::import_from_moo2() ... self.__is_outpost = %i" % self.__is_outpost)
        self.__morale		= lbx.read_char(data,  0x007) * 5 # Morale value is stored as divided by 5
        self.__pollution	= lbx.read_short_int(data, 0x008)
        self.set_population(lbx.read_char(data,  0x00a))
        print("     Colony::import_from_moo2() ... self.__population = %i" % self.get_population())
        self.assignment		= lbx.read_char(data,  0x00b)
#			0x00 = Agricultural Colony
#			0x01 = Industrial Colony
#			0x02 = Research Colony
#			0xff = (balanced?) Colony
#		0x00c ~ 0x0b3		colonists
	self.colonists		= read_population(data, self.get_population())
	self.__pop_raised		= [
				    lbx.read_short_int(data, 0x0B4),	# race 0
				    lbx.read_short_int(data, 0x0B6),	# race 1
				    lbx.read_short_int(data, 0x0B8),	# race 2
				    lbx.read_short_int(data, 0x0BA),	# race 3
				    lbx.read_short_int(data, 0x0BC),	# race 4
				    lbx.read_short_int(data, 0x0BE),	# race 5
				    lbx.read_short_int(data, 0x0C0),	# race 6
				    lbx.read_short_int(data, 0x0C2),	# race 7
				    lbx.read_short_int(data, 0x0C4),	# androids
				    lbx.read_short_int(data, 0x0C6)	# natives
				]
	self.__pop_grow		= [
				    lbx.read_short_int(data, 0x0C8),	# race 0
				    lbx.read_short_int(data, 0x0CA),	# race 1
				    lbx.read_short_int(data, 0x0CC),	# race 2
				    lbx.read_short_int(data, 0x0CE),	# race 3
				    lbx.read_short_int(data, 0x0D0),	# race 4
				    lbx.read_short_int(data, 0x0D2),	# race 5
				    lbx.read_short_int(data, 0x0D4),	# race 6
				    lbx.read_short_int(data, 0x0D6),	# race 7
				    lbx.read_short_int(data, 0x0D8),	# androids
				    lbx.read_short_int(data, 0x0DA)	# natives
				]
	self.n_turns_existed		= lbx.read_char(data,  0x0DC)	# bookeeping
	self.food2_per_farmer		= lbx.read_char(data,  0x0DD)	# Food per farmer in half-units of food
	self.industry_per_worker	= lbx.read_char(data,  0x0DE)
	self.research_per_scientist	= lbx.read_char(data,  0x0DF)
	self.max_farms			= lbx.read_char(data,  0x0E0)
	self.__max_population		= lbx.read_char(data,  0x0E1)
	self.climate			= lbx.read_char(data,  0x0E2)
	self.ground_strength		= lbx.read_short_int(data, 0x0E3)	# calculated for ai
	self.space_strength		= lbx.read_short_int(data, 0x0E5)	# calculated for ai
	self.set_food(lbx.read_short_int(data, 0x0E7))	# total food = food - population
	self.set_industry(lbx.read_short_int(data, 0x0E9))
	self.set_research(lbx.read_short_int(data, 0x0EB))
#		0x0ed		?

	self.__build_queue = [
	    {
		'item':		lbx.read_char(data,  0x115),
		'flags':	lbx.read_char(data,  0x116)
	    },
	    {
		'item':		lbx.read_char(data,  0x117),
		'flags':	lbx.read_char(data,  0x118)
	    },
	    {
		'item':		lbx.read_char(data,  0x119),
		'flags':	lbx.read_char(data,  0x11a)
	    },
	    {
		'item':		lbx.read_char(data,  0x11b),
		'flags':	lbx.read_char(data,  0x11c)
	    },
	    {
		'item':		lbx.read_char(data,  0x11d),
		'flags':	lbx.read_char(data,  0x11e)
	    },
	    {
		'item':		lbx.read_char(data,  0x11f),
		'flags':	lbx.read_char(data,  0x120)
	    },
	    {
		'item':		lbx.read_char(data,  0x121),
		'flags':	lbx.read_char(data,  0x122)
	    }
	]
#		0x115		building item #0				# 0x0b = colony base??? 0xf9 = spy? 0xfd = housing 0xfe = trade goods
#			0x0b = colony base
#			0xf9 = spy
#			0xfd = housing
#			0xfe = trade goods
#			0xff = nothing
#		0x116		?
#		0x117		building item #1				# 0x0b = colony base??? 0xf9 = spy?
#		0x118		?
#		0x119		building item #2				# 0x0b = colony base??? 0xf9 = spy?
#		0x11a		?
#		0x11b		building item #3				# 0x0b = colony base??? 0xf9 = spy?
#		0x11c		?
#		0x11d		building item #4				# 0x0b = colony base??? 0xf9 = spy?
#		0x11e		?
#		0x11f		building item #5				# 0x0b = colony base??? 0xf9 = spy?
#		0x120		?
#		0x121		building item #6				# 0x0b = colony base??? 0xf9 = spy?
#		0x122		?
	self.marines			= lbx.read_char(data, 0x130)
	self.armors			= lbx.read_char(data, 0x132)
	self.__buildings			= []
	for b_id in range(1, 49):
    	    offset = 0x136 + b_id
#            self.buildings.append(ord(data[offset]))
	    if ord(data[offset]):
		self.__buildings.append(b_id)

#	self.print_info()

    def set_population(self, population):
	self.__population = population

    def get_population(self):
        return self.__population

    def list_buildings(self):
	return self.__buildings

    def has_building(self, b_id):
	return b_id in self.__buildings

    def assign_planet(self, planet):
	self.__planet = planet

    def get_food_summary(self):
        return self.__food_summary

    def get_industry_summary(self):
        return self.__industry_summary

    def get_research_summary(self):
        return self.__research_summary

    def get_morale_summary(self):
        return self.__morale_summary

    def get_bc_summary(self):
        return self.__bc_summary

    """
    """
    def display_summary(self, title, foot, summary):
        total = rules.count_summary_result(summary)
	print("+----------------------------------------------------+")
	print("+ %s +" % title.ljust(50))
	print("+ ================================================== +")
        for k in summary:
            if summary[k]:
                print("+ %s ... %s +" % (str(summary[k]).rjust(6), k.ljust(39)))
	print("+                                                    +")
        print("+ % 6i ... %s +" % (total, foot.ljust(39)))
	print("+----------------------------------------------------+")

    """
	Returns Morale Summary
    """
    def print_morale_summary(self):
        self.display_summary("Morale Summary", "Total", self.get_morale_summary())
    # end func print_morale_summary

    def print_bc_summary(self):
        self.display_summary("BC Summary", "Total Income", self.get_bc_summary())
    # end func print_bc_summary

    def print_food_summary(self):
        self.display_summary("Food Summary", "Total Food Produced", self.get_food_summary())
    # end func print_food_summary

    def print_industry_summary(self):
        self.display_summary("Industry Summary", "Total Industry Produced", self.get_industry_summary())
    # end func print_industry_summary

    def print_research_summary(self):
        self.display_summary("Research Summary", "Research Industry Produced", self.get_research_summary())
    # end func print_research_summary

    """
	counts pollution for given production
    """
    def xget_colony_pollution(self, production, PLAYERS):
	planet = self.__planet
	if self.is_outpost():
	    return 0

#	print "$$$ get_colony_pollution $$$"

	if self.has_building(B_CORE_WASTE_DUMP):
	    return 0

	production = float(production)

	tolerant = 0
	pop = 0

	for t in (FARMER, SCIENTIST, WORKER):
	    for colonist in self.colonists[t]:
		pop += 1
		if PLAYERS[colonist['race']]['racepicks']['tolerant']:
		    tolerant += 1

#	print "    population: %i" % pop
#	print "    tolerant: %i" % tolerant
#	print "    non-tolerant: %i" % (pop - tolerant)
#	print "    planet size: %i" % planet['size']

#	print "	production before atm. renewer: %s" % str(production)
	if self.has_building(B_ATMOSPHERE_RENEWER):
	    production = math.ceil(production / 4)
#	print "	production after atm. renewer: %s" % str(production)

	tolerance = [2, 4, 6, 8, 10][planet['size']]

#	if PLAYERS[self.get_owner()]['technologies'][TECH_NANO_DISASSEMBLERS] == 3:
	if TECH_NANO_DISASSEMBLERS in PLAYERS[self.get_owner()]['known_techs']:
	    tolerance += tolerance

#	print "    planet tolerance: %i" % tolerance
	pollution = float(max(0, production - tolerance)) / 2
#	print "    pollution: %s" % str(pollution)

	pollution = float(pop - tolerant) * pollution / float(pop)
#	print "    pollution: %s" % str(pollution)

	return round(pollution)
    # end func get_colony_pollution

    def exists(self):
        return self.get_owner() < 0xff

    def set_owner(self, player_id):
	self.__owner = player_id

    def get_owner(self):
	return self.__owner

    def is_owned_by(self, player_id):
        return self.get_owner() == player_id

    def is_outpost(self):
        return self.__is_outpost
	
    def is_colony(self):
	return not self.is_outpost()

    def max_population(self):
	return self.__max_population

    def get_officer(self):
        return self.__officer

    def morale(self):
	return self.__morale

    def bc(self):
	return self.__bc

    def set_food(self, food):
	self.__food = food

    def get_food(self):
	return self.__food

    def set_industry(self, industry):
	self.__industry = industry

    def get_industry(self):
	return self.__industry

    def planet(self):
	return self.__planet

    def pollution(self):
	return self.__pollution

    def set_research(self, research):
	self.__research = research

    def get_research(self):
	return self.__research

    def pop_raised(self):
	return self.__pop_raised

    def pop_grow(self):
	return self.__pop_grow

    def build_queue(self):
	return self.__build_queue

    """
	get_agregated_populations
    """
    def get_agregated_populations(self):
	pops    = [0, 0, 0, 0, 0, 0, 0, 0]
        for t in [FARMER, WORKER, SCIENTIST]:
	    for colonist in self.colonists[t]:
		pops[colonist['race']] += 1
	return pops
    # end func get_agregated_populations


    def get_max_populations(self):
        return self.__max_populations

    """
    http://masteroforion2.blogspot.com/2005/09/growth-formula.html
    """
    def get_population_growth(self, max_populations, PLAYERS):
	growth = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
	if self.is_outpost():
	    return growth

	pops   = self.get_agregated_populations()

#	print "$$$ get_population_growth $$$ ... %s" % self.name
#	print "		populations: %s" % pops
#	print "		max_populations: %s" % max_populations


	total_population = sum(pops)

#	microbiotics	= False
#    	universal_antidote	= False

#    	print PLAYERS[colony['owner']]['technologies']
#    	print "	universal_antidote: %i" % PLAYERS[colony['owner']]['technologies'][193]

	if 192 in PLAYERS[self.get_owner()].get_known_techs():
	    universal_antidote_bonus	= 50
	else:
	    universal_antidote_bonus	= 0

	for race in range(8):
	    race_population = pops[race]
	    if race_population:
#    		print "	race: %i" % race
		max_population = max_populations[race]
#    		print "		race_population: %i" % race_population
#		print "		max_population: %i" % max_population
#		print "		total_population: %i" % total_population
#		print "		universal_antidote_bonus: %i" % universal_antidote_bonus

		b = math.floor((2000 * race_population * max(0, max_population - total_population) / max_population) ** 0.5)

		g, t, r, l, h = 0, 0, 0, 0, 0

		race_bonus			= PLAYERS[race].get_racepick_item('population')	# most races have 0, Sakkra have 100 here
		microbiotics_bonus		= 0	# 25 when invented
		random_bonus		= 0	# 100 when Boom of 100%
		leader_bonus		= 0	# e.g. 30 on Medicine 30%

		a = 100 + race_bonus + (microbiotics_bonus + universal_antidote_bonus) + random_bonus + leader_bonus

    # TODO: apply Clonning center
		c = 0

		growth[race] = int(((a * b) / 100) + math.floor(c))
    
#		print "$$$ get_population_growth $$$"
#		print "	race_population: %i" % race_population
#		print "	max_population: %i" % max_population
#		print "	agregated_population: %i" % agregated_population
#		print "	=> b: %i" % b
#		print "	+"
#		print "	a ..."
#		print "	="
#		print "	%i" % growth

	return growth
    # end func get_population_growth

    """
	raise the population
    """
    def raise_population(self):
	if self.get_owner() == 0xff:
	    print "	owner is 0xff"
	    return

	max_populations = self.get_max_populations()
#	print "raise_population ... max_populations: %s" % str(max_populations)
#	print "raise_population ... pop_raised: %s" % str(self.__pop_raised)
#	print "raise_population ... pop_grow: %s" % str(self.__pop_grow)
	for race in range(8):
    	    self.__pop_raised[race] += self.__pop_grow[race]
    	    if self.__pop_raised[race] > 999:
		# the race that turns over 999 in pop_raised gets a new farmer
        	r1 = self.get_owner()
                self.colonists[FARMER].append({'a': r1 & race, 'b': FARMER, 'c': 0x00, 'd': 0x00, 'r1': r1, 'race': race})
                self.__pop_raised[race] -= 1000
                self.__population += 1
    # end func raise_population

    def total_population(self):
        return len(self.colonists[FARMER]) + len(self.colonists[WORKER]) + len(self.colonists[SCIENTIST])

    def sumary_result(self, summary):
        res = 0.0
        for k, v in summary.iterkeys():
                res += float(v)
        return res

    """
	recounts colony values, should be called after any change
    """
    def recount(self, game_rules, colony_leader, players):
#	print "$$$ colony::recount() ... colony_id = %i" % self.colony_id
#	planet = GAME['DATA']['planets'][self.planet_id]
#	PLAYERS = GAME['DATA']['players']

	self.__bc			= 0
	if not self.exists():
            self.__population           = 0
            self.__max_populations      = rules.get_empty_max_populations()
#	    print "	owner is 0xff"
#	    print
	    return

#	planet = self.__planet

	if self.is_outpost():
#	    print "	this is outpost"
#	    print
            self.__population           = 0
	    self.__pollution		= 0
	    self.__industry		= 0
	    self.__research		= 0
#	    self.__food_summary		= {}
#	    self.__industry_summary	= {}
#	    self.__research_summary	= {}
            self.__max_populations      = rules.get_empty_max_populations()
	    return
	else:
	
#            officer = self.get_officer()
#            print(" * officer: %i" % officer)


#	    print "	name: %s" % self.name
#	    print "	owner: %s (%i)" % (PLAYERS[self.get_owner()]['race'], self.get_owner())
	
            

	    """
		morale
	    """
	    self.__morale_summary = rules.compose_morale_summary(game_rules, self, colony_leader, players)
	    self.__morale = rules.count_summary_result(self.__morale_summary)

	    """
		food
	    """
#	    print "### Recount of colony %i ... %s" % (self.colony_id, self.name)
#	    print "	industry:"
#	    print "		before: %i" % self.__industry
	    self.__food_summary = rules.compose_food_summary(game_rules, self, players)
	    self.__food = rules.count_summary_result(self.__food_summary)
#	    print "		after: %i" % self.__industry

	    """
		industry
	    """
#	    print "### Recount of colony %i ... %s" % (self.colony_id, self.name)
#	    print "	industry:"
#	    print "		before: %i" % self.__industry
	    self.__industry_summary = rules.compose_industry_summary(game_rules, self, colony_leader, players)
	    self.__pollution = self.__industry_summary['pollution']
	    self.__industry = rules.count_summary_result(self.__industry_summary)
#	    print "		after: %i" % self.__industry

            """
                BC
            """
	    self.__bc_summary = rules.compose_bc_summary(game_rules, self, players)
	    self.__bc = rules.count_summary_result(self.__bc_summary)

	    """
		research
	    """
#	    print "	research:"
#	    print "		before: %i" % self.__research
	    self.__research_summary = rules.compose_research_summary(game_rules, self, players)
	    self.__research = rules.count_summary_result(self.__research_summary)
#	    print "		after: %i" % self.__research

	    """
		population
	    """
	    self.__max_populations = rules.compose_max_populations(self, players)
	    self.__max_population = max(self.__max_populations)
	    self.__pop_grow = self.get_population_growth(self.__max_populations, players)

#	    for t in [FARMER, WORKER, SCIENTIST]:
#		for i in range(len(self.colonists[t])):
#		    pass
#		    print "colonists type #%i ... race: %i" % (t, self.colonists[t][i]['race'])
#		for colonist in self.colonists[t]:
#            	pops[colonist['race']] += 1

#	    print
#	    print("")
    # end func recount

class EnemyColony(Colony):

    def __init__(self, colony_id, player_id):
        self.set_id(colony_id)
        self.set_owner(player_id)

    def is_outpost(self):
        return False